# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: test/v1/simple.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'test/v1/simple.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api.expr.v1alpha1 import checked_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_checked__pb2
from google.api.expr.v1alpha1 import eval_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_eval__pb2
from google.api.expr.v1alpha1 import value_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14test/v1/simple.proto\x12\x17google.api.expr.test.v1\x1a&google/api/expr/v1alpha1/checked.proto\x1a#google/api/expr/v1alpha1/eval.proto\x1a$google/api/expr/v1alpha1/value.proto\"p\n\x0eSimpleTestFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12;\n\x07section\x18\x03 \x03(\x0b\x32*.google.api.expr.test.v1.SimpleTestSection\"i\n\x11SimpleTestSection\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x31\n\x04test\x18\x03 \x03(\x0b\x32#.google.api.expr.test.v1.SimpleTest\"\xe4\x05\n\nSimpleTest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04\x65xpr\x18\x03 \x01(\t\x12\x16\n\x0e\x64isable_macros\x18\x04 \x01(\x08\x12\x15\n\rdisable_check\x18\x05 \x01(\x08\x12\x12\n\ncheck_only\x18\x0f \x01(\x08\x12\x30\n\x08type_env\x18\x06 \x03(\x0b\x32\x1e.google.api.expr.v1alpha1.Decl\x12\x11\n\tcontainer\x18\r \x01(\t\x12\x43\n\x08\x62indings\x18\x07 \x03(\x0b\x32\x31.google.api.expr.test.v1.SimpleTest.BindingsEntry\x12\x30\n\x05value\x18\x08 \x01(\x0b\x32\x1f.google.api.expr.v1alpha1.ValueH\x00\x12<\n\x0ctyped_result\x18\x10 \x01(\x0b\x32$.google.api.expr.test.v1.TypedResultH\x00\x12\x38\n\neval_error\x18\t \x01(\x0b\x32\".google.api.expr.v1alpha1.ErrorSetH\x00\x12\x43\n\x0f\x61ny_eval_errors\x18\n \x01(\x0b\x32(.google.api.expr.test.v1.ErrorSetMatcherH\x00\x12\x37\n\x07unknown\x18\x0b \x01(\x0b\x32$.google.api.expr.v1alpha1.UnknownSetH\x00\x12\x42\n\x0c\x61ny_unknowns\x18\x0c \x01(\x0b\x32*.google.api.expr.test.v1.UnknownSetMatcherH\x00\x1aT\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.google.api.expr.v1alpha1.ExprValue:\x02\x38\x01\x42\x10\n\x0eresult_matcherJ\x04\x08\x0e\x10\x0f\"t\n\x0bTypedResult\x12/\n\x06result\x18\x01 \x01(\x0b\x32\x1f.google.api.expr.v1alpha1.Value\x12\x34\n\x0c\x64\x65\x64uced_type\x18\x02 \x01(\x0b\x32\x1e.google.api.expr.v1alpha1.Type\"E\n\x0f\x45rrorSetMatcher\x12\x32\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\".google.api.expr.v1alpha1.ErrorSet\"K\n\x11UnknownSetMatcher\x12\x36\n\x08unknowns\x18\x01 \x03(\x0b\x32$.google.api.expr.v1alpha1.UnknownSetBG\n\x1b\x63om.google.api.expr.test.v1B\x0bSimpleProtoZ\x1b\x63\x65l.dev/expr/test/v1/testpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'test.v1.simple_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033com.google.api.expr.test.v1B\013SimpleProtoZ\033cel.dev/expr/test/v1/testpb'
  _globals['_SIMPLETEST_BINDINGSENTRY']._loaded_options = None
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_options = b'8\001'
  _globals['_SIMPLETESTFILE']._serialized_start=164
  _globals['_SIMPLETESTFILE']._serialized_end=276
  _globals['_SIMPLETESTSECTION']._serialized_start=278
  _globals['_SIMPLETESTSECTION']._serialized_end=383
  _globals['_SIMPLETEST']._serialized_start=386
  _globals['_SIMPLETEST']._serialized_end=1126
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_start=1018
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_end=1102
  _globals['_TYPEDRESULT']._serialized_start=1128
  _globals['_TYPEDRESULT']._serialized_end=1244
  _globals['_ERRORSETMATCHER']._serialized_start=1246
  _globals['_ERRORSETMATCHER']._serialized_end=1315
  _globals['_UNKNOWNSETMATCHER']._serialized_start=1317
  _globals['_UNKNOWNSETMATCHER']._serialized_end=1392
# @@protoc_insertion_point(module_scope)



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests 'utf-8' encoded error message."""

import unittest
import weakref

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_UNICODE_ERROR_MESSAGES = [
    b"\xe2\x80\x9d".decode("utf-8"),
    b"abc\x80\xd0\xaf".decode("latin-1"),
    b"\xc3\xa9".decode("utf-8"),
]

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, request_streaming=None, response_streaming=None):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None

    def unary_unary(self, request, servicer_context):
        servicer_context.set_code(grpc.StatusCode.UNKNOWN)
        servicer_context.set_details(request.decode("utf-8"))
        return _RESPONSE


_METHOD_HANDLERS = {_UNARY_UNARY: _MethodHandler()}


class ErrorMessageEncodingTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, _METHOD_HANDLERS
        )
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

    def tearDown(self):
        self._server.stop(0)
        self._channel.close()

    def testMessageEncoding(self):
        for message in _UNICODE_ERROR_MESSAGES:
            multi_callable = self._channel.unary_unary(
                grpc._common.fully_qualified_method(
                    _SERVICE_NAME, _UNARY_UNARY
                ),
                _registered_method=True,
            )
            with self.assertRaises(grpc.RpcError) as cm:
                multi_callable(message.encode("utf-8"))

            self.assertEqual(cm.exception.code(), grpc.StatusCode.UNKNOWN)
            self.assertEqual(cm.exception.details(), message)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/trace/v3/lightstep.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/trace/v3/lightstep.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%envoy/config/trace/v3/lightstep.proto\x12\x15\x65nvoy.config.trace.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xed\x02\n\x0fLightstepConfig\x12\"\n\x11\x63ollector_cluster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12&\n\x11\x61\x63\x63\x65ss_token_file\x18\x02 \x01(\tB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x36\n\x0c\x61\x63\x63\x65ss_token\x18\x04 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\x12`\n\x11propagation_modes\x18\x03 \x03(\x0e\x32\x36.envoy.config.trace.v3.LightstepConfig.PropagationModeB\r\xfa\x42\n\x92\x01\x07\"\x05\x82\x01\x02\x10\x01\"F\n\x0fPropagationMode\x12\t\n\x05\x45NVOY\x10\x00\x12\r\n\tLIGHTSTEP\x10\x01\x12\x06\n\x02\x42\x33\x10\x02\x12\x11\n\rTRACE_CONTEXT\x10\x03:,\x9a\xc5\x88\x1e\'\n%envoy.config.trace.v2.LightstepConfigB\xb7\x01\n#io.envoyproxy.envoy.config.trace.v3B\x0eLightstepProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3\xf2\x98\xfe\x8f\x05,\x12*envoy.extensions.tracers.lightstep.v4alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.trace.v3.lightstep_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#io.envoyproxy.envoy.config.trace.v3B\016LightstepProtoP\001ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3\362\230\376\217\005,\022*envoy.extensions.tracers.lightstep.v4alpha\272\200\310\321\006\002\020\002'
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['collector_cluster']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['collector_cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['access_token_file']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['access_token_file']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['propagation_modes']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['propagation_modes']._serialized_options = b'\372B\n\222\001\007\"\005\202\001\002\020\001'
  _globals['_LIGHTSTEPCONFIG']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG']._serialized_options = b'\232\305\210\036\'\n%envoy.config.trace.v2.LightstepConfig'
  _globals['_LIGHTSTEPCONFIG']._serialized_start=258
  _globals['_LIGHTSTEPCONFIG']._serialized_end=623
  _globals['_LIGHTSTEPCONFIG_PROPAGATIONMODE']._serialized_start=507
  _globals['_LIGHTSTEPCONFIG_PROPAGATIONMODE']._serialized_end=577
# @@protoc_insertion_point(module_scope)



# Copyright 2026 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Regression test: py_grpc_library must work when the proto_library source is
a generated file (not a checked-in source file).

See https://github.com/grpc/grpc/issues/41792.
"""

import unittest

import gen_echo_pb2
import gen_echo_pb2_grpc


class GeneratedProtoImportTest(unittest.TestCase):
    def test_stubs_are_importable(self):
        from gen_echo_pb2_grpc import EchoServiceServicer
        from gen_echo_pb2_grpc import EchoServiceStub
        from gen_echo_pb2_grpc import add_EchoServiceServicer_to_server

    def test_proto_message_is_importable(self):
        from gen_echo_pb2 import EchoRequest


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: xds/type/matcher/v3/cel.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'xds/type/matcher/v3/cel.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.type.v3 import cel_pb2 as xds_dot_type_dot_v3_dot_cel__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dxds/type/matcher/v3/cel.proto\x12\x13xds.type.matcher.v3\x1a\x15xds/type/v3/cel.proto\x1a\x17validate/validate.proto\"[\n\nCelMatcher\x12\x38\n\nexpr_match\x18\x01 \x01(\x0b\x32\x1a.xds.type.v3.CelExpressionB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\tBX\n\x1e\x63om.github.xds.type.matcher.v3B\x08\x43\x65lProtoP\x01Z*github.com/cncf/xds/go/xds/type/matcher/v3b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'xds.type.matcher.v3.cel_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\036com.github.xds.type.matcher.v3B\010CelProtoP\001Z*github.com/cncf/xds/go/xds/type/matcher/v3'
  _globals['_CELMATCHER'].fields_by_name['expr_match']._loaded_options = None
  _globals['_CELMATCHER'].fields_by_name['expr_match']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CELMATCHER']._serialized_start=102
  _globals['_CELMATCHER']._serialized_end=193
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests behavior around the Core channel arguments."""

import asyncio
import errno
import logging
import platform
import random
import unittest

import grpc
from grpc.experimental import aio
import typeguard

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework import common
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_RANDOM_SEED = 42

_ENABLE_REUSE_PORT = "SO_REUSEPORT enabled"
_DISABLE_REUSE_PORT = "SO_REUSEPORT disabled"
_SOCKET_OPT_SO_REUSEPORT = "grpc.so_reuseport"
_OPTIONS = (
    (_ENABLE_REUSE_PORT, ((_SOCKET_OPT_SO_REUSEPORT, 1),)),
    (_DISABLE_REUSE_PORT, ((_SOCKET_OPT_SO_REUSEPORT, 0),)),
)

_NUM_SERVER_CREATED = 5

_GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH = "grpc.max_receive_message_length"
_MAX_MESSAGE_LENGTH = 1024

_ADDRESS_TOKEN_ERRNO = errno.EADDRINUSE, errno.ENOSR


class _TestPointerWrapper:
    def __int__(self):
        return 123456


_TEST_CHANNEL_ARGS = [
    ("arg1", b"bytes_val"),
    ("arg2", "str_val"),
    ("arg3", 1),
    (b"arg4", "str_val"),
    ("arg6", _TestPointerWrapper()),
]

_INVALID_TEST_CHANNEL_ARGS = [
    {"foo": "bar"},
    (("key",),),
    "str",
]


async def test_if_reuse_port_enabled(server: aio.Server):
    port = server.add_insecure_port("localhost:0")
    await server.start()

    try:
        with common.bound_socket(
            bind_address="localhost",
            port=port,
            listen=False,
        ) as (unused_host, bound_port):
            assert bound_port == port
    except OSError as e:
        if e.errno in _ADDRESS_TOKEN_ERRNO:
            return False
        else:
            logging.exception(e)
            raise
    else:
        return True


class TestChannelArgument(AioTestBase):
    async def setUp(self):
        random.seed(_RANDOM_SEED)

    @unittest.skipIf(
        platform.system() == "Windows",
        "SO_REUSEPORT only available in Linux-like OS.",
    )
    @unittest.skipIf(
        "aarch64" in platform.machine(),
        "SO_REUSEPORT needs to be enabled in Core's port.h.",
    )
    async def test_server_so_reuse_port_is_set_properly(self):
        async def test_body():
            fact, options = random.choice(_OPTIONS)
            server = aio.server(options=options)
            try:
                result = await test_if_reuse_port_enabled(server)
                if fact == _ENABLE_REUSE_PORT and not result:
                    self.fail(
                        "Enabled reuse port in options, but not observed in"
                        " socket"
                    )
                elif fact == _DISABLE_REUSE_PORT and result:
                    self.fail(
                        "Disabled reuse port in options, but observed in socket"
                    )
            finally:
                await server.stop(None)

        # Creating a lot of servers concurrently
        await asyncio.gather(*(test_body() for _ in range(_NUM_SERVER_CREATED)))

    async def test_client(self):
        # Do not segfault, or raise exception!
        channel = aio.insecure_channel("[::]:0", options=_TEST_CHANNEL_ARGS)
        await channel.close()

    async def test_server(self):
        # Do not segfault, or raise exception!
        server = aio.server(options=_TEST_CHANNEL_ARGS)
        await server.stop(None)

    @typeguard.suppress_type_checks  # testing negative cases
    async def test_invalid_client_args(self):
        for invalid_arg in _INVALID_TEST_CHANNEL_ARGS:
            self.assertRaises(
                (ValueError, TypeError),
                aio.insecure_channel,
                "[::]:0",
                options=invalid_arg,
            )

    async def test_max_message_length_applied(self):
        address, server = await start_test_server()

        async with aio.insecure_channel(
            address,
            options=(
                (_GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH, _MAX_MESSAGE_LENGTH),
            ),
        ) as channel:
            stub = test_pb2_grpc.TestServiceStub(channel)

            request = messages_pb2.StreamingOutputCallRequest()
            # First request will pass
            request.response_parameters.append(
                messages_pb2.ResponseParameters(
                    size=_MAX_MESSAGE_LENGTH // 2,
                )
            )
            # Second request should fail
            request.response_parameters.append(
                messages_pb2.ResponseParameters(
                    size=_MAX_MESSAGE_LENGTH * 2,
                )
            )

            call = stub.StreamingOutputCall(request)

            response = await call.read()
            self.assertEqual(
                _MAX_MESSAGE_LENGTH // 2, len(response.payload.body)
            )

            with self.assertRaises(aio.AioRpcError) as exception_context:
                await call.read()
            rpc_error = exception_context.exception
            self.assertEqual(
                grpc.StatusCode.RESOURCE_EXHAUSTED, rpc_error.code()
            )
            self.assertIn(str(_MAX_MESSAGE_LENGTH), rpc_error.details())

            self.assertEqual(
                grpc.StatusCode.RESOURCE_EXHAUSTED, await call.code()
            )

        await server.stop(None)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/vcl/v3alpha/vcl_socket_interface.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/vcl/v3alpha/vcl_socket_interface.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?contrib/envoy/extensions/vcl/v3alpha/vcl_socket_interface.proto\x12\x1c\x65nvoy.extensions.vcl.v3alpha\x1a\x1dudpa/annotations/status.proto\"\x14\n\x12VclSocketInterfaceB\x9e\x01\n*io.envoyproxy.envoy.extensions.vcl.v3alphaB\x17VclSocketInterfaceProtoP\x01ZKgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/vcl/v3alpha\xba\x80\xc8\xd1\x06\x04\x08\x01\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.vcl.v3alpha.vcl_socket_interface_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n*io.envoyproxy.envoy.extensions.vcl.v3alphaB\027VclSocketInterfaceProtoP\001ZKgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/vcl/v3alpha\272\200\310\321\006\004\010\001\020\002'
  _globals['_VCLSOCKETINTERFACE']._serialized_start=128
  _globals['_VCLSOCKETINTERFACE']._serialized_end=148
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import socket
import subprocess
import sys
import tempfile
import time
from urllib import request

sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import jobset

# must be synchronized with test/core/test_util/port_server_client.h
_PORT_SERVER_PORT = 32766


def start_port_server(verbose=False):
    # check if a compatible port server is running
    # if incompatible (version mismatch) ==> start a new one
    # if not running ==> start a new one
    # otherwise, leave it up
    try:
        version = int(
            request.urlopen(
                "http://localhost:%d/version_number" % _PORT_SERVER_PORT
            ).read()
        )
        logging.info("detected port server running version %d", version)
        running = True
    except Exception as e:
        if verbose:
            logging.exception("failed to detect port server")
        running = False
    if running:
        current_version = int(
            subprocess.check_output(
                [
                    sys.executable,  # use the same python binary as this process
                    os.path.abspath(
                        "tools/run_tests/python_utils/port_server.py"
                    ),
                    "dump_version",
                ]
            ).decode()
        )
        logging.info("my port server is version %d", current_version)
        running = version >= current_version
        if not running:
            logging.info("port_server version mismatch: killing the old one")
            request.urlopen(
                "http://localhost:%d/quitquitquit" % _PORT_SERVER_PORT
            ).read()
            time.sleep(1)
    if not running:
        fd, logfile = tempfile.mkstemp()
        os.close(fd)
        logging.info("starting port_server, with log file %s", logfile)
        args = [
            sys.executable,
            os.path.abspath("tools/run_tests/python_utils/port_server.py"),
            "-p",
            "%d" % _PORT_SERVER_PORT,
            "-l",
            logfile,
        ]
        env = dict(os.environ)
        env["BUILD_ID"] = "pleaseDontKillMeJenkins"
        if jobset.platform_string() == "windows":
            # Working directory of port server needs to be outside of Jenkins
            # workspace to prevent file lock issues.
            tempdir = tempfile.mkdtemp()
            # 0x00000008 DETACHED_PROCESS doesn't seem to work with python3
            creationflags = 0
            port_server = subprocess.Popen(
                args,
                env=env,
                cwd=tempdir,
                creationflags=creationflags,
                close_fds=True,
            )
        else:
            port_server = subprocess.Popen(
                args, env=env, preexec_fn=os.setsid, close_fds=True
            )
        time.sleep(1)
        # ensure port server is up
        waits = 0
        while True:
            if waits > 10:
                logging.warning(
                    "killing port server due to excessive start up waits"
                )
                port_server.kill()
            if port_server.poll() is not None:
                logging.error("port_server failed to start")
                # try one final time: maybe another build managed to start one
                time.sleep(1)
                try:
                    request.urlopen(
                        "http://localhost:%d/get" % _PORT_SERVER_PORT
                    ).read()
                    logging.info(
                        "last ditch attempt to contact port server succeeded"
                    )
                    break
                except:
                    logging.exception(
                        "final attempt to contact port server failed"
                    )
                    port_log = open(logfile, "r").read()
                    print(port_log)
                    sys.exit(1)
            try:
                port_server_url = "http://localhost:%d/get" % _PORT_SERVER_PORT
                request.urlopen(port_server_url).read()
                logging.info("port server is up and ready")
                break
            except socket.timeout:
                logging.exception("while waiting for port_server")
                time.sleep(1)
                waits += 1
            except IOError:
                logging.exception("while waiting for port_server")
                time.sleep(1)
                waits += 1
            except:
                logging.exception(
                    (
                        'error while contacting port server at "%s".'
                        "Will try killing it."
                    ),
                    port_server_url,
                )
                port_server.kill()
                raise



#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Makes DNS queries for A records to specified servers"""

import argparse
import threading
import time

import twisted.internet.reactor as reactor
import twisted.internet.task as task
import twisted.names.client as client


def main():
    argp = argparse.ArgumentParser(description="Make DNS queries for A records")
    argp.add_argument(
        "-s",
        "--server_host",
        default="127.0.0.1",
        type=str,
        help="Host for DNS server to listen on for TCP and UDP.",
    )
    argp.add_argument(
        "-p",
        "--server_port",
        default=53,
        type=int,
        help="Port that the DNS server is listening on.",
    )
    argp.add_argument(
        "-n",
        "--qname",
        default=None,
        type=str,
        help="Name of the record to query for. ",
    )
    argp.add_argument(
        "-t",
        "--timeout",
        default=1,
        type=int,
        help="Force process exit after this number of seconds.",
    )
    args = argp.parse_args()

    def OnResolverResultAvailable(result):
        answers, authority, additional = result
        for a in answers:
            print(a.payload)

    def BeginQuery(reactor, qname):
        servers = [(args.server_host, args.server_port)]
        resolver = client.Resolver(servers=servers)
        deferred_result = resolver.lookupAddress(args.qname)
        deferred_result.addCallback(OnResolverResultAvailable)
        return deferred_result

    task.react(BeginQuery, [args.qname])


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/bootstrap/reverse_tunnel/downstream_socket_interface/v3/downstream_reverse_connection_socket_interface.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/bootstrap/reverse_tunnel/downstream_socket_interface/v3/downstream_reverse_connection_socket_interface.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n}envoy/extensions/bootstrap/reverse_tunnel/downstream_socket_interface/v3/downstream_reverse_connection_socket_interface.proto\x12Henvoy.extensions.bootstrap.reverse_tunnel.downstream_socket_interface.v3\x1a\x1dudpa/annotations/status.proto\"`\n*DownstreamReverseConnectionSocketInterface\x12\x13\n\x0bstat_prefix\x18\x01 \x01(\t\x12\x1d\n\x15\x65nable_detailed_stats\x18\x02 \x01(\x08\x42\xa3\x02\nVio.envoyproxy.envoy.extensions.bootstrap.reverse_tunnel.downstream_socket_interface.v3B/DownstreamReverseConnectionSocketInterfaceProtoP\x01Z\x8d\x01github.com/envoyproxy/go-control-plane/envoy/extensions/bootstrap/reverse_tunnel/downstream_socket_interface/v3;downstream_socket_interfacev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.bootstrap.reverse_tunnel.downstream_socket_interface.v3.downstream_reverse_connection_socket_interface_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nVio.envoyproxy.envoy.extensions.bootstrap.reverse_tunnel.downstream_socket_interface.v3B/DownstreamReverseConnectionSocketInterfaceProtoP\001Z\215\001github.com/envoyproxy/go-control-plane/envoy/extensions/bootstrap/reverse_tunnel/downstream_socket_interface/v3;downstream_socket_interfacev3\272\200\310\321\006\002\020\002'
  _globals['_DOWNSTREAMREVERSECONNECTIONSOCKETINTERFACE']._serialized_start=234
  _globals['_DOWNSTREAMREVERSECONNECTIONSOCKETINTERFACE']._serialized_end=330
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/credential_injector/v3/credential_injector.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/credential_injector/v3/credential_injector.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nNenvoy/extensions/filters/http/credential_injector/v3/credential_injector.proto\x12\x34\x65nvoy.extensions.filters.http.credential_injector.v3\x1a$envoy/config/core/v3/extension.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9b\x01\n\x12\x43redentialInjector\x12\x11\n\toverwrite\x18\x01 \x01(\x08\x12(\n allow_request_without_credential\x18\x02 \x01(\x08\x12H\n\ncredential\x18\x03 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\xda\x01\nBio.envoyproxy.envoy.extensions.filters.http.credential_injector.v3B\x17\x43redentialInjectorProtoP\x01Zqgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/credential_injector/v3;credential_injectorv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.credential_injector.v3.credential_injector_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nBio.envoyproxy.envoy.extensions.filters.http.credential_injector.v3B\027CredentialInjectorProtoP\001Zqgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/credential_injector/v3;credential_injectorv3\272\200\310\321\006\002\020\002'
  _globals['_CREDENTIALINJECTOR'].fields_by_name['credential']._loaded_options = None
  _globals['_CREDENTIALINJECTOR'].fields_by_name['credential']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CREDENTIALINJECTOR']._serialized_start=231
  _globals['_CREDENTIALINJECTOR']._serialized_end=386
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import grpc_testing

from tests.testing.proto import requests_pb2
from tests.testing.proto import services_pb2

# TODO(https://github.com/grpc/grpc/issues/11657): Eliminate this entirely.
# TODO(https://github.com/protocolbuffers/protobuf/issues/3452): Eliminate this if/else.
if services_pb2.DESCRIPTOR.services_by_name.get("FirstService") is None:
    FIRST_SERVICE = "Fix protobuf issue 3452!"
    FIRST_SERVICE_UNUN = "Fix protobuf issue 3452!"
    FIRST_SERVICE_UNSTRE = "Fix protobuf issue 3452!"
    FIRST_SERVICE_STREUN = "Fix protobuf issue 3452!"
    FIRST_SERVICE_STRESTRE = "Fix protobuf issue 3452!"
else:
    FIRST_SERVICE = services_pb2.DESCRIPTOR.services_by_name["FirstService"]
    FIRST_SERVICE_UNUN = FIRST_SERVICE.methods_by_name["UnUn"]
    FIRST_SERVICE_UNSTRE = FIRST_SERVICE.methods_by_name["UnStre"]
    FIRST_SERVICE_STREUN = FIRST_SERVICE.methods_by_name["StreUn"]
    FIRST_SERVICE_STRESTRE = FIRST_SERVICE.methods_by_name["StreStre"]



# Copyright 2018 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Channelz debug service implementation in gRPC Python."""

import sys

import grpc
from grpc_channelz.v1._servicer import ChannelzServicer
import grpc_channelz.v1.channelz_pb2_grpc as _channelz_pb2_grpc

_add_channelz_servicer_doc = """Add Channelz servicer to a server.

Channelz servicer is in charge of
pulling information from C-Core for entire process. It will allow the
server to response to Channelz queries.

The Channelz statistic is enabled by default inside C-Core. Whether the
statistic is enabled or not is isolated from adding Channelz servicer.
That means you can query Channelz info with a Channelz-disabled channel,
and you can add Channelz servicer to a Channelz-disabled server.

The Channelz statistic can be enabled or disabled by channel option
'grpc.enable_channelz'. Set to 1 to enable, set to 0 to disable.

This is an EXPERIMENTAL API.

Args:
    server: A gRPC server to which Channelz service will be added.
"""

if sys.version_info[0] >= 3 and sys.version_info[1] >= 6:
    from grpc_channelz.v1 import _async as aio

    def add_channelz_servicer(server):
        if isinstance(server, grpc.experimental.aio.Server):
            _channelz_pb2_grpc.add_ChannelzServicer_to_server(
                aio.ChannelzServicer(), server
            )
        else:
            _channelz_pb2_grpc.add_ChannelzServicer_to_server(
                ChannelzServicer(), server
            )

    add_channelz_servicer.__doc__ = _add_channelz_servicer_doc

    __all__ = [
        "ChannelzServicer",
        "add_channelz_servicer",
        "aio",
    ]

else:

    def add_channelz_servicer(server):
        _channelz_pb2_grpc.add_ChannelzServicer_to_server(
            ChannelzServicer(), server
        )

    add_channelz_servicer.__doc__ = _add_channelz_servicer_doc

    __all__ = [
        "ChannelzServicer",
        "add_channelz_servicer",
    ]



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc._channel.Channel connectivity."""

import logging
import threading
import time
import unittest

import grpc

from tests.unit import thread_pool
from tests.unit.framework.common import test_constants


def _ready_in_connectivities(connectivities):
    return grpc.ChannelConnectivity.READY in connectivities


def _last_connectivity_is_not_ready(connectivities):
    return connectivities[-1] is not grpc.ChannelConnectivity.READY


class _Callback:
    def __init__(self):
        self._condition = threading.Condition()
        self._connectivities = []

    def update(self, connectivity):
        with self._condition:
            self._connectivities.append(connectivity)
            self._condition.notify()

    def connectivities(self):
        with self._condition:
            return tuple(self._connectivities)

    def block_until_connectivities_satisfy(self, predicate):
        with self._condition:
            while True:
                connectivities = tuple(self._connectivities)
                if predicate(connectivities):
                    return connectivities
                else:
                    self._condition.wait()


class ChannelConnectivityTest(unittest.TestCase):
    def test_lonely_channel_connectivity(self):
        callback = _Callback()

        channel = grpc.insecure_channel("localhost:12345")
        channel.subscribe(callback.update, try_to_connect=False)
        first_connectivities = callback.block_until_connectivities_satisfy(bool)
        channel.subscribe(callback.update, try_to_connect=True)
        second_connectivities = callback.block_until_connectivities_satisfy(
            lambda connectivities: 2 <= len(connectivities)
        )
        # Wait for a connection that will never happen.
        time.sleep(test_constants.SHORT_TIMEOUT)
        third_connectivities = callback.connectivities()
        channel.unsubscribe(callback.update)
        fourth_connectivities = callback.connectivities()
        channel.unsubscribe(callback.update)
        fifth_connectivities = callback.connectivities()

        channel.close()

        self.assertSequenceEqual(
            (grpc.ChannelConnectivity.IDLE,), first_connectivities
        )
        self.assertNotIn(grpc.ChannelConnectivity.READY, second_connectivities)
        self.assertNotIn(grpc.ChannelConnectivity.READY, third_connectivities)
        self.assertNotIn(grpc.ChannelConnectivity.READY, fourth_connectivities)
        self.assertNotIn(grpc.ChannelConnectivity.READY, fifth_connectivities)

    def test_immediately_connectable_channel_connectivity(self):
        recording_thread_pool = thread_pool.RecordingThreadPool(
            max_workers=None
        )
        server = grpc.server(
            recording_thread_pool, options=(("grpc.so_reuseport", 0),)
        )
        port = server.add_insecure_port("[::]:0")
        server.start()
        first_callback = _Callback()
        second_callback = _Callback()

        channel = grpc.insecure_channel("localhost:{}".format(port))
        channel.subscribe(first_callback.update, try_to_connect=False)
        first_connectivities = (
            first_callback.block_until_connectivities_satisfy(bool)
        )
        # Wait for a connection that will never happen because try_to_connect=True
        # has not yet been passed.
        time.sleep(test_constants.SHORT_TIMEOUT)
        second_connectivities = first_callback.connectivities()
        channel.subscribe(second_callback.update, try_to_connect=True)
        third_connectivities = (
            first_callback.block_until_connectivities_satisfy(
                lambda connectivities: 2 <= len(connectivities)
            )
        )
        fourth_connectivities = (
            second_callback.block_until_connectivities_satisfy(bool)
        )
        # Wait for a connection that will happen (or may already have happened).
        first_callback.block_until_connectivities_satisfy(
            _ready_in_connectivities
        )
        second_callback.block_until_connectivities_satisfy(
            _ready_in_connectivities
        )
        channel.close()
        server.stop(None)

        self.assertSequenceEqual(
            (grpc.ChannelConnectivity.IDLE,), first_connectivities
        )
        self.assertSequenceEqual(
            (grpc.ChannelConnectivity.IDLE,), second_connectivities
        )
        self.assertNotIn(
            grpc.ChannelConnectivity.TRANSIENT_FAILURE, third_connectivities
        )
        self.assertNotIn(
            grpc.ChannelConnectivity.SHUTDOWN, third_connectivities
        )
        self.assertNotIn(
            grpc.ChannelConnectivity.TRANSIENT_FAILURE, fourth_connectivities
        )
        self.assertNotIn(
            grpc.ChannelConnectivity.SHUTDOWN, fourth_connectivities
        )
        self.assertFalse(recording_thread_pool.was_used())

    def test_reachable_then_unreachable_channel_connectivity(self):
        recording_thread_pool = thread_pool.RecordingThreadPool(
            max_workers=None
        )
        server = grpc.server(
            recording_thread_pool, options=(("grpc.so_reuseport", 0),)
        )
        port = server.add_insecure_port("[::]:0")
        server.start()
        callback = _Callback()

        channel = grpc.insecure_channel("localhost:{}".format(port))
        channel.subscribe(callback.update, try_to_connect=True)
        callback.block_until_connectivities_satisfy(_ready_in_connectivities)
        # Now take down the server and confirm that channel readiness is repudiated.
        server.stop(None)
        callback.block_until_connectivities_satisfy(
            _last_connectivity_is_not_ready
        )
        channel.unsubscribe(callback.update)
        channel.close()
        self.assertFalse(recording_thread_pool.was_used())


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/conformance/test/simple.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/conformance/test/simple.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr import checked_pb2 as cel_dot_expr_dot_checked__pb2
from cel.expr import eval_pb2 as cel_dot_expr_dot_eval__pb2
from cel.expr import value_pb2 as cel_dot_expr_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&cel/expr/conformance/test/simple.proto\x12\x19\x63\x65l.expr.conformance.test\x1a\x16\x63\x65l/expr/checked.proto\x1a\x13\x63\x65l/expr/eval.proto\x1a\x14\x63\x65l/expr/value.proto\"r\n\x0eSimpleTestFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12=\n\x07section\x18\x03 \x03(\x0b\x32,.cel.expr.conformance.test.SimpleTestSection\"k\n\x11SimpleTestSection\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x33\n\x04test\x18\x03 \x03(\x0b\x32%.cel.expr.conformance.test.SimpleTest\"\xa6\x05\n\nSimpleTest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04\x65xpr\x18\x03 \x01(\t\x12\x16\n\x0e\x64isable_macros\x18\x04 \x01(\x08\x12\x15\n\rdisable_check\x18\x05 \x01(\x08\x12\x12\n\ncheck_only\x18\x0f \x01(\x08\x12 \n\x08type_env\x18\x06 \x03(\x0b\x32\x0e.cel.expr.Decl\x12\x11\n\tcontainer\x18\r \x01(\t\x12\x0e\n\x06locale\x18\x0e \x01(\t\x12\x45\n\x08\x62indings\x18\x07 \x03(\x0b\x32\x33.cel.expr.conformance.test.SimpleTest.BindingsEntry\x12 \n\x05value\x18\x08 \x01(\x0b\x32\x0f.cel.expr.ValueH\x00\x12>\n\x0ctyped_result\x18\x10 \x01(\x0b\x32&.cel.expr.conformance.test.TypedResultH\x00\x12(\n\neval_error\x18\t \x01(\x0b\x32\x12.cel.expr.ErrorSetH\x00\x12\x45\n\x0f\x61ny_eval_errors\x18\n \x01(\x0b\x32*.cel.expr.conformance.test.ErrorSetMatcherH\x00\x12\'\n\x07unknown\x18\x0b \x01(\x0b\x32\x14.cel.expr.UnknownSetH\x00\x12\x44\n\x0c\x61ny_unknowns\x18\x0c \x01(\x0b\x32,.cel.expr.conformance.test.UnknownSetMatcherH\x00\x1a\x44\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.cel.expr.ExprValue:\x02\x38\x01\x42\x10\n\x0eresult_matcher\"T\n\x0bTypedResult\x12\x1f\n\x06result\x18\x01 \x01(\x0b\x32\x0f.cel.expr.Value\x12$\n\x0c\x64\x65\x64uced_type\x18\x02 \x01(\x0b\x32\x0e.cel.expr.Type\"5\n\x0f\x45rrorSetMatcher\x12\"\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\x12.cel.expr.ErrorSet\";\n\x11UnknownSetMatcher\x12&\n\x08unknowns\x18\x01 \x03(\x0b\x32\x14.cel.expr.UnknownSetBP\n\x1d\x64\x65v.cel.expr.conformance.testB\x0bSimpleProtoP\x01Z\x1d\x63\x65l.dev/expr/conformance/test\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.conformance.test.simple_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\035dev.cel.expr.conformance.testB\013SimpleProtoP\001Z\035cel.dev/expr/conformance/test\370\001\001'
  _globals['_SIMPLETEST_BINDINGSENTRY']._loaded_options = None
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_options = b'8\001'
  _globals['_SIMPLETESTFILE']._serialized_start=136
  _globals['_SIMPLETESTFILE']._serialized_end=250
  _globals['_SIMPLETESTSECTION']._serialized_start=252
  _globals['_SIMPLETESTSECTION']._serialized_end=359
  _globals['_SIMPLETEST']._serialized_start=362
  _globals['_SIMPLETEST']._serialized_end=1040
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_start=954
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_end=1022
  _globals['_TYPEDRESULT']._serialized_start=1042
  _globals['_TYPEDRESULT']._serialized_end=1126
  _globals['_ERRORSETMATCHER']._serialized_start=1128
  _globals['_ERRORSETMATCHER']._serialized_end=1181
  _globals['_UNKNOWNSETMATCHER']._serialized_start=1183
  _globals['_UNKNOWNSETMATCHER']._serialized_end=1242
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/load_balancing_policies/client_side_weighted_round_robin/v3/client_side_weighted_round_robin.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/load_balancing_policies/client_side_weighted_round_robin/v3/client_side_weighted_round_robin.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.extensions.load_balancing_policies.common.v3 import common_pb2 as envoy_dot_extensions_dot_load__balancing__policies_dot_common_dot_v3_dot_common__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nsenvoy/extensions/load_balancing_policies/client_side_weighted_round_robin/v3/client_side_weighted_round_robin.proto\x12Lenvoy.extensions.load_balancing_policies.client_side_weighted_round_robin.v3\x1a?envoy/extensions/load_balancing_policies/common/v3/common.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x99\x04\n\x1c\x43lientSideWeightedRoundRobin\x12:\n\x16\x65nable_oob_load_report\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x37\n\x14oob_reporting_period\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x32\n\x0f\x62lackout_period\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12;\n\x18weight_expiration_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x37\n\x14weight_update_period\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12J\n\x19\x65rror_utilization_penalty\x18\x06 \x01(\x0b\x32\x1b.google.protobuf.FloatValueB\n\xfa\x42\x07\n\x05-\x00\x00\x00\x00\x12.\n&metric_names_for_computing_utilization\x18\x07 \x03(\t\x12^\n\x11slow_start_config\x18\x08 \x01(\x0b\x32\x43.envoy.extensions.load_balancing_policies.common.v3.SlowStartConfigB\xa2\x02\nZio.envoyproxy.envoy.extensions.load_balancing_policies.client_side_weighted_round_robin.v3B!ClientSideWeightedRoundRobinProtoP\x01Z\x96\x01github.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/client_side_weighted_round_robin/v3;client_side_weighted_round_robinv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.load_balancing_policies.client_side_weighted_round_robin.v3.client_side_weighted_round_robin_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nZio.envoyproxy.envoy.extensions.load_balancing_policies.client_side_weighted_round_robin.v3B!ClientSideWeightedRoundRobinProtoP\001Z\226\001github.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/client_side_weighted_round_robin/v3;client_side_weighted_round_robinv3\272\200\310\321\006\002\020\002'
  _globals['_CLIENTSIDEWEIGHTEDROUNDROBIN'].fields_by_name['error_utilization_penalty']._loaded_options = None
  _globals['_CLIENTSIDEWEIGHTEDROUNDROBIN'].fields_by_name['error_utilization_penalty']._serialized_options = b'\372B\007\n\005-\000\000\000\000'
  _globals['_CLIENTSIDEWEIGHTEDROUNDROBIN']._serialized_start=383
  _globals['_CLIENTSIDEWEIGHTEDROUNDROBIN']._serialized_end=920
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interceptor that ensures a specific header is present."""

import grpc


def _unary_unary_rpc_terminator(code, details):
    def terminate(ignored_request, context):
        context.abort(code, details)

    return grpc.unary_unary_rpc_method_handler(terminate)


class RequestHeaderValidatorInterceptor(grpc.ServerInterceptor):
    def __init__(self, header, value, code, details):
        self._header = header
        self._value = value
        self._terminator = _unary_unary_rpc_terminator(code, details)

    def intercept_service(self, continuation, handler_call_details):
        if (
            self._header,
            self._value,
        ) in handler_call_details.invocation_metadata:
            return continuation(handler_call_details)
        else:
            return self._terminator



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests clean shutdown of server on various interpreter exit conditions.

The tests in this module spawn a subprocess for each test case, the
test is considered successful if it doesn't freeze/timeout.
"""

import atexit
import logging
import os
import subprocess
import sys
import threading
import unittest

from tests.unit import _server_shutdown_scenarios

SCENARIO_FILE = os.path.abspath(
    os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        "_server_shutdown_scenarios.py",
    )
)
INTERPRETER = sys.executable
BASE_COMMAND = [INTERPRETER, SCENARIO_FILE]

processes = []
process_lock = threading.Lock()


# Make sure we attempt to clean up any
# processes we may have left running
def cleanup_processes():
    with process_lock:
        for process in processes:
            try:
                process.kill()
            except Exception:  # pylint: disable=broad-except
                pass


atexit.register(cleanup_processes)


def wait(process):
    with process_lock:
        processes.append(process)
    process.wait()


class ServerShutdown(unittest.TestCase):
    # Currently we shut down a server (if possible) after the Python server
    # instance is garbage collected. This behavior may change in the future.
    def test_deallocated_server_stops(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_server_shutdown_scenarios.SERVER_DEALLOCATED],
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        wait(process)

    def test_server_exception_exits(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_server_shutdown_scenarios.SERVER_RAISES_EXCEPTION],
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        wait(process)

    @unittest.skipIf(os.name == "nt", "fork not supported on windows")
    def test_server_fork_can_exit(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_server_shutdown_scenarios.SERVER_FORK_CAN_EXIT],
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        wait(process)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc.beta.utilities."""

import threading
import time
import unittest

from grpc.beta import implementations
from grpc.beta import utilities
from grpc.framework.foundation import future

from tests.unit.framework.common import test_constants


class _Callback:
    def __init__(self):
        self._condition = threading.Condition()
        self._value = None

    def accept_value(self, value):
        with self._condition:
            self._value = value
            self._condition.notify_all()

    def block_until_called(self):
        with self._condition:
            while self._value is None:
                self._condition.wait()
            return self._value


@unittest.skip("https://github.com/grpc/grpc/issues/16134")
class ChannelConnectivityTest(unittest.TestCase):
    def test_lonely_channel_connectivity(self):
        channel = implementations.insecure_channel("localhost", 12345)
        callback = _Callback()

        ready_future = utilities.channel_ready_future(channel)
        ready_future.add_done_callback(callback.accept_value)
        with self.assertRaises(future.TimeoutError):
            ready_future.result(timeout=test_constants.SHORT_TIMEOUT)
        self.assertFalse(ready_future.cancelled())
        self.assertFalse(ready_future.done())
        self.assertTrue(ready_future.running())
        ready_future.cancel()
        value_passed_to_callback = callback.block_until_called()
        self.assertIs(ready_future, value_passed_to_callback)
        self.assertTrue(ready_future.cancelled())
        self.assertTrue(ready_future.done())
        self.assertFalse(ready_future.running())

    def test_immediately_connectable_channel_connectivity(self):
        server = implementations.server({})
        port = server.add_insecure_port("[::]:0")
        server.start()
        channel = implementations.insecure_channel("localhost", port)
        callback = _Callback()

        try:
            ready_future = utilities.channel_ready_future(channel)
            ready_future.add_done_callback(callback.accept_value)
            self.assertIsNone(
                ready_future.result(timeout=test_constants.LONG_TIMEOUT)
            )
            value_passed_to_callback = callback.block_until_called()
            self.assertIs(ready_future, value_passed_to_callback)
            self.assertFalse(ready_future.cancelled())
            self.assertTrue(ready_future.done())
            self.assertFalse(ready_future.running())
            # Cancellation after maturity has no effect.
            ready_future.cancel()
            self.assertFalse(ready_future.cancelled())
            self.assertTrue(ready_future.done())
            self.assertFalse(ready_future.running())
        finally:
            ready_future.cancel()
            server.stop(0)


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# Copyright 2022 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helps with running bazel with extra settings to generate structured test reports in CI."""

import argparse
import os
import platform
import sys
import uuid

sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import utils_common

_ROOT = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))
os.chdir(_ROOT)

# How long to sleep before querying Resultstore API and uploading to bigquery
# (to let ResultStore finish writing results from the bazel invocation that has
# just finished).
_UPLOAD_RBE_RESULTS_DELAY_SECONDS = 60


def _platform_string():
    """Detect current platform"""
    if platform.system() == "Windows":
        return "windows"
    elif platform.system()[:7] == "MSYS_NT":
        return "windows"
    elif platform.system() == "Darwin":
        return "mac"
    elif platform.system() == "Linux":
        return "linux"
    else:
        return "posix"


def _append_to_kokoro_bazel_invocations(invocation_id: str) -> None:
    """Kokoro can display "Bazel" result link on kokoro jobs if told so."""
    # to get "bazel" link for kokoro build, we need to upload
    # the "bazel_invocation_ids" file with bazel invocation ID as artifact.
    kokoro_artifacts_dir = os.getenv("KOKORO_ARTIFACTS_DIR")
    if kokoro_artifacts_dir:
        # append the bazel invocation UUID to the bazel_invocation_ids file.
        with open(
            os.path.join(kokoro_artifacts_dir, "bazel_invocation_ids"), "a"
        ) as f:
            f.write(invocation_id + "\n")
        print(
            'Added invocation ID %s to kokoro "bazel_invocation_ids" artifact'
            % invocation_id,
            file=sys.stderr,
        )
    else:
        print(
            'Skipped adding invocation ID %s to kokoro "bazel_invocation_ids"'
            " artifact" % invocation_id,
            file=sys.stderr,
        )
        pass


def _generate_junit_report_string(
    report_suite_name: str, invocation_id: str, success: bool
) -> None:
    """Generate sponge_log.xml formatted report, that will make the bazel invocation reachable as a target in resultstore UI / sponge."""
    bazel_invocation_url = (
        "https://source.cloud.google.com/results/invocations/%s" % invocation_id
    )
    package_name = report_suite_name
    # set testcase name to invocation URL. That way, the link will be displayed in some form
    # resultstore UI and sponge even in case the bazel invocation succeeds.
    testcase_name = bazel_invocation_url
    if success:
        # unfortunately, neither resultstore UI nor sponge display the "system-err" output (or any other tags)
        # on a passing test case. But at least we tried.
        test_output_tag = (
            "<system-err>PASSED. See invocation results here: %s</system-err>"
            % bazel_invocation_url
        )
    else:
        # The failure output will be displayed in both resultstore UI and sponge when clicking on the failing testcase.
        test_output_tag = (
            '<failure message="Failure">FAILED. See bazel invocation results'
            " here: %s</failure>" % bazel_invocation_url
        )

    lines = [
        "<testsuites>",
        '<testsuite id="1" name="%s" package="%s">'
        % (report_suite_name, package_name),
        '<testcase name="%s">' % testcase_name,
        test_output_tag,
        "</testcase></testsuite>",
        "</testsuites>",
    ]
    return "\n".join(lines)


def _create_bazel_wrapper(
    report_path: str,
    report_suite_name: str,
    invocation_id: str,
    upload_results: bool,
) -> None:
    """Create a "bazel wrapper" script that will execute bazel with extra settings and postprocessing."""

    os.makedirs(report_path, exist_ok=True)

    bazel_wrapper_filename = os.path.join(report_path, "bazel_wrapper")
    bazel_wrapper_bat_filename = bazel_wrapper_filename + ".bat"
    bazel_rc_filename = os.path.join(report_path, "bazel_wrapper.bazelrc")

    # put xml reports in a separate directory if requested by GRPC_TEST_REPORT_BASE_DIR
    report_base_dir = os.getenv("GRPC_TEST_REPORT_BASE_DIR", None)
    xml_report_path = os.path.abspath(
        os.path.join(report_base_dir, report_path)
        if report_base_dir
        else report_path
    )
    os.makedirs(xml_report_path, exist_ok=True)

    failing_report_filename = os.path.join(xml_report_path, "sponge_log.xml")
    success_report_filename = os.path.join(
        xml_report_path, "success_log_to_rename.xml"
    )

    # invoking "bash" explicitly is important for the workspace status command
    # to work on windows as well.
    workspace_status_command = (
        "bash tools/remote_build/workspace_status_kokoro.sh"
    )

    # generate RC file with the bazel flags we want to use apply.
    # Using an RC file solves problems with flag ordering in the wrapper.
    # (e.g. some flags need to come after the build/test command)
    with open(bazel_rc_filename, "w") as f:
        f.write('build --invocation_id="%s"\n' % invocation_id)
        f.write(
            'build --workspace_status_command="%s"\n' % workspace_status_command
        )

    # generate "failing" and "success" report
    # the "failing" is named as "sponge_log.xml", which is the name picked up by sponge/resultstore
    # so the failing report will be used by default (unless we later replace the report with
    # one that says "success"). That way if something goes wrong before bazel is run,
    # there will at least be a "failing" target that indicates that (we really don't want silent failures).
    with open(failing_report_filename, "w") as f:
        f.write(
            _generate_junit_report_string(
                report_suite_name, invocation_id, success=False
            )
        )
    with open(success_report_filename, "w") as f:
        f.write(
            _generate_junit_report_string(
                report_suite_name, invocation_id, success=True
            )
        )

    # generate the bazel wrapper for linux/macos
    with open(bazel_wrapper_filename, "w") as f:
        intro_lines = [
            "#!/bin/bash",
            "set -ex",
            "",
            'tools/bazel --bazelrc="%s" "$@" || FAILED=true'
            % bazel_rc_filename,
            "",
        ]

        if upload_results:
            upload_results_lines = [
                "sleep %s" % _UPLOAD_RBE_RESULTS_DELAY_SECONDS,
                "PYTHONHTTPSVERIFY=0 python3"
                " ./tools/run_tests/python_utils/upload_rbe_results.py"
                ' --invocation_id="%s"' % invocation_id,
                "",
            ]
        else:
            upload_results_lines = []

        outro_lines = [
            'if [ "$FAILED" != "" ]',
            "then",
            "  exit 1",
            "else",
            (
                "  # success: plant the pre-generated xml report that says"
                ' "success"'
            ),
            "  mv -f %s %s"
            % (success_report_filename, failing_report_filename),
            "fi",
        ]

        lines = [
            line + "\n"
            for line in intro_lines + upload_results_lines + outro_lines
        ]
        f.writelines(lines)
    os.chmod(bazel_wrapper_filename, 0o775)  # make the unix wrapper executable

    # generate bazel wrapper for windows
    with open(bazel_wrapper_bat_filename, "w") as f:
        intro_lines = [
            "@echo on",
            "",
            'bazel --bazelrc="%s" %%*' % bazel_rc_filename,
            "set BAZEL_EXITCODE=%errorlevel%",
            "",
        ]

        if upload_results:
            upload_results_lines = [
                "sleep %s" % _UPLOAD_RBE_RESULTS_DELAY_SECONDS,
                "python3 tools/run_tests/python_utils/upload_rbe_results.py"
                ' --invocation_id="%s" || exit /b 1' % invocation_id,
                "",
            ]
        else:
            upload_results_lines = []

        outro_lines = [
            "if %BAZEL_EXITCODE% == 0 (",
            (
                "  @rem success: plant the pre-generated xml report that says"
                ' "success"'
            ),
            "  mv -f %s %s"
            % (success_report_filename, failing_report_filename),
            ")",
            "exit /b %BAZEL_EXITCODE%",
        ]

        lines = [
            line + "\n"
            for line in intro_lines + upload_results_lines + outro_lines
        ]
        f.writelines(lines)

    print("Bazel invocation ID: %s" % invocation_id, file=sys.stderr)
    print(
        "Upload test results to BigQuery after bazel runs: %s" % upload_results,
        file=sys.stderr,
    )
    print(
        "Generated bazel wrapper: %s" % bazel_wrapper_filename, file=sys.stderr
    )
    print(
        "Generated bazel wrapper: %s" % bazel_wrapper_bat_filename,
        file=sys.stderr,
    )


if __name__ == "__main__":
    # parse command line
    argp = argparse.ArgumentParser(
        description=(
            "Generate bazel wrapper to help with bazel test reports in CI."
        )
    )
    argp.add_argument(
        "--report_path",
        required=True,
        type=str,
        help=(
            "Path under which the bazel wrapper and other files are going to be"
            " generated"
        ),
    )
    argp.add_argument(
        "--report_suite_name",
        default="bazel_invocations",
        type=str,
        help="Test suite name to use in generated XML report",
    )
    args = argp.parse_args()

    # generate new bazel invocation ID
    invocation_id = str(uuid.uuid4())

    report_path = args.report_path
    report_suite_name = args.report_suite_name
    upload_results = True if os.getenv("UPLOAD_TEST_RESULTS") else False

    if upload_results:
        # Only upload results when initiated by the CI.
        upload_results = utils_common.should_upload_results_on_ci()

    _append_to_kokoro_bazel_invocations(invocation_id)
    _create_bazel_wrapper(
        report_path, report_suite_name, invocation_id, upload_results
    )



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import collections
import logging
import multiprocessing
import os
import sys
import time
from typing import Tuple

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import benchmark_service_pb2_grpc
from src.proto.grpc.testing import control_pb2
from src.proto.grpc.testing import stats_pb2
from src.proto.grpc.testing import worker_service_pb2_grpc
from tests.qps import histogram
from tests.unit import resources
from tests.unit.framework.common import get_socket
from tests_aio.benchmark import benchmark_client
from tests_aio.benchmark import benchmark_servicer

_NUM_CORES = multiprocessing.cpu_count()
_WORKER_ENTRY_FILE = os.path.join(
    os.path.split(os.path.abspath(__file__))[0], "worker.py"
)

_LOGGER = logging.getLogger(__name__)


class _SubWorker(
    collections.namedtuple("_SubWorker", ["process", "port", "channel", "stub"])
):
    """A data class that holds information about a child qps worker."""

    def _repr(self):
        return f"<_SubWorker pid={self.process.pid} port={self.port}>"

    def __repr__(self):
        return self._repr()

    def __str__(self):
        return self._repr()


def _get_server_status(
    start_time: float, end_time: float, port: int
) -> control_pb2.ServerStatus:
    """Creates ServerStatus proto message."""
    end_time = time.monotonic()
    elapsed_time = end_time - start_time
    # TODO(lidiz) Collect accurate time system to compute QPS/core-second.
    stats = stats_pb2.ServerStats(
        time_elapsed=elapsed_time,
        time_user=elapsed_time,
        time_system=elapsed_time,
    )
    return control_pb2.ServerStatus(stats=stats, port=port, cores=_NUM_CORES)


def _create_server(config: control_pb2.ServerConfig) -> Tuple[aio.Server, int]:
    """Creates a server object according to the ServerConfig."""
    channel_args = tuple(
        (
            (arg.name, arg.str_value)
            if arg.HasField("str_value")
            else (arg.name, int(arg.int_value))
        )
        for arg in config.channel_args
    )

    server = aio.server(options=channel_args + (("grpc.so_reuseport", 1),))
    if config.server_type == control_pb2.ASYNC_SERVER:
        servicer = benchmark_servicer.BenchmarkServicer()
        benchmark_service_pb2_grpc.add_BenchmarkServiceServicer_to_server(
            servicer, server
        )
    elif config.server_type == control_pb2.ASYNC_GENERIC_SERVER:
        resp_size = config.payload_config.bytebuf_params.resp_size
        servicer = benchmark_servicer.GenericBenchmarkServicer(resp_size)
        method_implementations = {
            "StreamingCall": grpc.stream_stream_rpc_method_handler(
                servicer.StreamingCall
            ),
            "UnaryCall": grpc.unary_unary_rpc_method_handler(
                servicer.UnaryCall
            ),
        }
        handler = grpc.method_handlers_generic_handler(
            "grpc.testing.BenchmarkService", method_implementations
        )
        server.add_generic_rpc_handlers((handler,))
    else:
        raise NotImplementedError(
            "Unsupported server type {}".format(config.server_type)
        )

    if config.HasField("security_params"):  # Use SSL
        server_creds = grpc.ssl_server_credentials(
            ((resources.private_key(), resources.certificate_chain()),)
        )
        port = server.add_secure_port(
            "[::]:{}".format(config.port), server_creds
        )
    else:
        port = server.add_insecure_port("[::]:{}".format(config.port))

    return server, port


def _get_client_status(
    start_time: float, end_time: float, qps_data: histogram.Histogram
) -> control_pb2.ClientStatus:
    """Creates ClientStatus proto message."""
    latencies = qps_data.get_data()
    end_time = time.monotonic()
    elapsed_time = end_time - start_time
    # TODO(lidiz) Collect accurate time system to compute QPS/core-second.
    stats = stats_pb2.ClientStats(
        latencies=latencies,
        time_elapsed=elapsed_time,
        time_user=elapsed_time,
        time_system=elapsed_time,
    )
    return control_pb2.ClientStatus(stats=stats)


def _create_client(
    server: str, config: control_pb2.ClientConfig, qps_data: histogram.Histogram
) -> benchmark_client.BenchmarkClient:
    """Creates a client object according to the ClientConfig."""
    if config.load_params.WhichOneof("load") != "closed_loop":
        raise NotImplementedError(
            f"Unsupported load parameter {config.load_params}"
        )

    if config.client_type == control_pb2.ASYNC_CLIENT:
        if config.rpc_type == control_pb2.UNARY:
            client_type = benchmark_client.UnaryAsyncBenchmarkClient
        elif config.rpc_type == control_pb2.STREAMING:
            client_type = benchmark_client.StreamingAsyncBenchmarkClient
        elif config.rpc_type == control_pb2.STREAMING_FROM_SERVER:
            client_type = benchmark_client.ServerStreamingAsyncBenchmarkClient
        else:
            raise NotImplementedError(
                f"Unsupported rpc_type [{config.rpc_type}]"
            )
    else:
        raise NotImplementedError(
            f"Unsupported client type {config.client_type}"
        )

    return client_type(server, config, qps_data)


def _pick_an_unused_port() -> int:
    """Picks an unused TCP port."""
    _, port, sock = get_socket()
    sock.close()
    return port


async def _create_sub_worker() -> _SubWorker:
    """Creates a child qps worker as a subprocess."""
    port = _pick_an_unused_port()

    _LOGGER.info("Creating sub worker at port [%d]...", port)
    process = await asyncio.create_subprocess_exec(
        sys.executable, _WORKER_ENTRY_FILE, "--driver_port", str(port)
    )
    _LOGGER.info(
        "Created sub worker process for port [%d] at pid [%d]",
        port,
        process.pid,
    )
    channel = aio.insecure_channel(f"localhost:{port}")
    _LOGGER.info("Waiting for sub worker at port [%d]", port)
    await channel.channel_ready()
    stub = worker_service_pb2_grpc.WorkerServiceStub(channel)
    return _SubWorker(
        process=process,
        port=port,
        channel=channel,
        stub=stub,
    )


class WorkerServicer(worker_service_pb2_grpc.WorkerServiceServicer):
    """Python Worker Server implementation."""

    def __init__(self):
        self._loop = asyncio.get_event_loop()
        self._quit_event = asyncio.Event()

    async def _run_single_server(self, config, request_iterator, context):
        server, port = _create_server(config)
        await server.start()
        _LOGGER.info("Server started at port [%d]", port)

        start_time = time.monotonic()
        await context.write(_get_server_status(start_time, start_time, port))

        async for request in request_iterator:
            end_time = time.monotonic()
            status = _get_server_status(start_time, end_time, port)
            if request.mark.reset:
                start_time = end_time
            await context.write(status)
        await server.stop(None)

    async def RunServer(self, request_iterator, context):
        config_request = await context.read()
        config = config_request.setup
        _LOGGER.info("Received ServerConfig: %s", config)

        if config.server_processes <= 0:
            _LOGGER.info("Using server_processes == [%d]", _NUM_CORES)
            config.server_processes = _NUM_CORES

        if config.port == 0:
            config.port = _pick_an_unused_port()
        _LOGGER.info("Port picked [%d]", config.port)

        if config.server_processes == 1:
            # If server_processes == 1, start the server in this process.
            await self._run_single_server(config, request_iterator, context)
        else:
            # If server_processes > 1, offload to other processes.
            sub_workers = await asyncio.gather(
                *[_create_sub_worker() for _ in range(config.server_processes)]
            )

            calls = [worker.stub.RunServer() for worker in sub_workers]

            config_request.setup.server_processes = 1

            for call in calls:
                await call.write(config_request)
                # An empty status indicates the peer is ready
                await call.read()

            start_time = time.monotonic()
            await context.write(
                _get_server_status(
                    start_time,
                    start_time,
                    config.port,
                )
            )

            _LOGGER.info("Servers are ready to serve.")

            async for request in request_iterator:
                end_time = time.monotonic()

                for call in calls:
                    await call.write(request)
                    # Reports from sub workers doesn't matter
                    await call.read()

                status = _get_server_status(
                    start_time,
                    end_time,
                    config.port,
                )
                if request.mark.reset:
                    start_time = end_time
                await context.write(status)

            for call in calls:
                await call.done_writing()

            for worker in sub_workers:
                await worker.stub.QuitWorker(control_pb2.Void())
                await worker.channel.close()
                _LOGGER.info("Waiting for [%s] to quit...", worker)
                await worker.process.wait()

    async def _run_single_client(self, config, request_iterator, context):
        running_tasks = []
        qps_data = histogram.Histogram(
            config.histogram_params.resolution,
            config.histogram_params.max_possible,
        )
        start_time = time.monotonic()

        # Create a client for each channel as asyncio.Task
        for i in range(config.client_channels):
            server = config.server_targets[i % len(config.server_targets)]
            client = _create_client(server, config, qps_data)
            _LOGGER.info("Client created against server [%s]", server)
            running_tasks.append(self._loop.create_task(client.run()))

        end_time = time.monotonic()
        await context.write(_get_client_status(start_time, end_time, qps_data))

        # Respond to stat requests
        async for request in request_iterator:
            end_time = time.monotonic()
            status = _get_client_status(start_time, end_time, qps_data)
            if request.mark.reset:
                qps_data.reset()
                start_time = time.monotonic()
            await context.write(status)

        # Cleanup the clients
        for task in running_tasks:
            task.cancel()

    async def RunClient(self, request_iterator, context):
        config_request = await context.read()
        config = config_request.setup
        _LOGGER.info("Received ClientConfig: %s", config)

        if config.client_processes <= 0:
            _LOGGER.info(
                "client_processes can't be [%d]", config.client_processes
            )
            _LOGGER.info("Using client_processes == [%d]", _NUM_CORES)
            config.client_processes = _NUM_CORES

        if config.client_processes == 1:
            # If client_processes == 1, run the benchmark in this process.
            await self._run_single_client(config, request_iterator, context)
        else:
            # If client_processes > 1, offload the work to other processes.
            sub_workers = await asyncio.gather(
                *[_create_sub_worker() for _ in range(config.client_processes)]
            )

            calls = [worker.stub.RunClient() for worker in sub_workers]

            config_request.setup.client_processes = 1

            for call in calls:
                await call.write(config_request)
                # An empty status indicates the peer is ready
                await call.read()

            start_time = time.monotonic()
            result = histogram.Histogram(
                config.histogram_params.resolution,
                config.histogram_params.max_possible,
            )
            end_time = time.monotonic()
            await context.write(
                _get_client_status(start_time, end_time, result)
            )

            async for request in request_iterator:
                end_time = time.monotonic()

                for call in calls:
                    _LOGGER.debug("Fetching status...")
                    await call.write(request)
                    sub_status = await call.read()
                    result.merge(sub_status.stats.latencies)
                    _LOGGER.debug(
                        "Update from sub worker count=[%d]",
                        sub_status.stats.latencies.count,
                    )

                status = _get_client_status(start_time, end_time, result)
                if request.mark.reset:
                    result.reset()
                    start_time = time.monotonic()
                _LOGGER.debug(
                    "Reporting count=[%d]", status.stats.latencies.count
                )
                await context.write(status)

            for call in calls:
                await call.done_writing()

            for worker in sub_workers:
                await worker.stub.QuitWorker(control_pb2.Void())
                await worker.channel.close()
                _LOGGER.info("Waiting for sub worker [%s] to quit...", worker)
                await worker.process.wait()
                _LOGGER.info("Sub worker [%s] quit", worker)

    @staticmethod
    async def CoreCount(unused_request, unused_context):
        return control_pb2.CoreResponse(cores=_NUM_CORES)

    async def QuitWorker(self, unused_request, unused_context):
        _LOGGER.info("QuitWorker command received.")
        self._quit_event.set()
        return control_pb2.Void()

    async def wait_for_quit(self):
        await self._quit_event.wait()



#!/usr/bin/env python3

# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import argparse
import errno
import filecmp
import glob
import os
import os.path
import pprint
import shutil
import subprocess
import sys
import traceback
import uuid

# the template for the content of protoc_lib_deps.py
DEPS_FILE_CONTENT = """
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# AUTO-GENERATED BY make_grpcio_tools.py!
CC_FILES={cc_files}

PROTO_FILES={proto_files}

CC_INCLUDES={cc_includes}
PROTO_INCLUDE={proto_include}

{commit_hash_expr}
"""

# expose commit hash suffix and prefix for check_grpcio_tools.py
COMMIT_HASH_PREFIX = 'PROTOBUF_SUBMODULE_VERSION="'
COMMIT_HASH_SUFFIX = '"'

EXTERNAL_LINKS = [
    ("@com_google_absl//", "third_party/abseil-cpp/"),
    ("@com_google_protobuf//", "third_party/protobuf/"),
    ("@utf8_range//", "third_party/protobuf/third_party/utf8_range"),
]

PROTOBUF_PROTO_PREFIX = "@com_google_protobuf//"

# Exclude conditional dependencies that are not needed by gRPC.
# Required files can be found at third_party/protobuf/src/file_lists.cmake
UPB_EXCLUDE_CC_FILES_PATTERN = "third_party/protobuf/upb/wire/decode_fast"
UPB_EXCLUDE_CC_FILES_EXCEPTIONS = [
    "third_party/protobuf/upb/wire/decode_fast/select.c",
]

# will be added to include path when building grpcio_tools
CC_INCLUDES = [
    os.path.join("third_party", "abseil-cpp"),
    os.path.join("third_party", "protobuf"),
    os.path.join("third_party", "protobuf", "src"),
    os.path.join("third_party", "protobuf", "upb_generator", "cmake"),
    os.path.join("third_party", "protobuf", "upb"),
    os.path.join("third_party", "protobuf", "upb", "reflection", "cmake"),
    os.path.join("third_party", "protobuf", "third_party", "utf8_range"),
]

# include path for .proto files
PROTO_INCLUDE = os.path.join("third_party", "protobuf")

# the target directory is relative to the grpcio_tools package root.
GRPCIO_TOOLS_ROOT_PREFIX = "tools/distrib/python/grpcio_tools/"

# Pairs of (source, target) directories to copy
# from the grpc repo root to the grpcio_tools build root.
COPY_FILES_SOURCE_TARGET_PAIRS = [
    ("include", "grpc_root/include"),
    ("src/compiler", "grpc_root/src/compiler"),
    ("third_party/abseil-cpp/absl", "third_party/abseil-cpp/absl"),
    ("third_party/protobuf/src", "third_party/protobuf/src"),
    ("third_party/protobuf/go", "third_party/protobuf/go"),
    ("third_party/protobuf/java", "third_party/protobuf/java"),
    ("third_party/protobuf/upb", "third_party/protobuf/upb"),
    (
        "third_party/protobuf/upb_generator",
        "third_party/protobuf/upb_generator",
    ),
    (
        "third_party/protobuf/third_party/utf8_range",
        "third_party/protobuf/third_party/utf8_range",
    ),
]

DELETE_TARGETS_ON_CLEANUP = ["third_party"]

# grpc repo root
GRPC_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "..", "..")
)

# the directory under which to probe for the current protobuf commit SHA
GRPC_PROTOBUF_SUBMODULE_ROOT = os.path.join(
    GRPC_ROOT, "third_party", "protobuf"
)

# the file to generate
GRPC_PYTHON_PROTOC_LIB_DEPS = os.path.join(
    GRPC_ROOT,
    "tools",
    "distrib",
    "python",
    "grpcio_tools",
    "protoc_lib_deps.py",
)

# the script to run for getting dependencies
BAZEL_DEPS = os.path.join(
    GRPC_ROOT, "tools", "distrib", "python", "bazel_deps.sh"
)

# the bazel target to scrape to get list of sources for the build
BAZEL_DEPS_PROTOC_LIB_QUERY = "@com_google_protobuf//:protoc_lib"

BAZEL_DEPS_COMMON_PROTOS_QUERIES = [
    "@com_google_protobuf//:well_known_type_protos",
    # has both plugin.proto and descriptor.proto
    "@com_google_protobuf//:compiler_plugin_proto",
    # language features protos
    "@com_google_protobuf//:cpp_features_proto",
    "@com_google_protobuf//:java_features_proto",
    "@com_google_protobuf//:go_features_proto",
]


def protobuf_submodule_commit_hash():
    """Gets the commit hash for the HEAD of the protobuf submodule currently
    checked out."""
    cwd = os.getcwd()
    os.chdir(GRPC_PROTOBUF_SUBMODULE_ROOT)
    output = subprocess.check_output(["git", "rev-parse", "HEAD"])
    os.chdir(cwd)
    return output.decode("ascii").splitlines()[0].strip()


def _bazel_query(query):
    """Runs 'bazel query' to collect source file info."""
    print('Running "bazel query %s"' % query)
    output = subprocess.check_output([BAZEL_DEPS, query])
    return output.decode("ascii").splitlines()


def _pretty_print_list(items):
    """Pretty print python list"""
    formatted = pprint.pformat(items, indent=4)
    # add newline after opening bracket (and fix indent of the next line)
    if formatted.startswith("["):
        formatted = formatted[0] + "\n " + formatted[1:]
    # add newline before closing bracket
    if formatted.endswith("]"):
        formatted = formatted[:-1] + "\n" + formatted[-1]
    return formatted


def _bazel_name_to_file_path(name):
    """Transform bazel reference to source file name."""
    for link in EXTERNAL_LINKS:
        if name.startswith(link[0]):
            filepath = link[1] + name[len(link[0]) :].replace(":", "/")

            # For some reason, the WKT sources (such as wrappers.pb.cc)
            # end up being reported by bazel as having an extra 'wkt/google/protobuf'
            # in path. Removing it makes the compilation pass.
            # TODO(jtattermusch) Get dir of this hack.
            filepath = filepath.replace("wkt/google/protobuf/", "")

            # The some files which Bazel generates during the build process,
            # are not accessible for Python build. Therefore, they need to be redirected
            # to use a pre-generated file for Cmake instead.
            filepath = filepath.replace(
                "/upb/reflection/stage1/", "/upb/reflection/cmake/"
            )
            filepath = filepath.replace(
                "/upb_generator/stage1/", "/upb_generator/cmake/"
            )
            if (
                "/upb/reflection/stage0/" in filepath
                or "/upb_generator/stage0/" in filepath
            ):
                return None

            # Unnecessary files that has main function need to be excluded.
            if (
                "/upb_generator/minitable/main.cc" in filepath
                or "/upb_generator/c/generator.cc" in filepath
                or "/protobuf/compiler/main_no_generators.cc" in filepath
            ):
                return None

            return filepath
    return None


def _generate_deps_file_content():
    """Returns the data structure with dependencies of protoc as python code."""
    cc_files_output = _bazel_query(BAZEL_DEPS_PROTOC_LIB_QUERY)

    # Collect .cc files (that will be later included in the native extension build)
    cc_files = []
    for name in cc_files_output:
        if name.endswith(".c") or name.endswith(".cc"):
            filepath = _bazel_name_to_file_path(name)
            if filepath:
                cc_files.append(filepath)

    # Exclude certain conditional dependencies that don't build properly
    # from upb.
    def _should_include(cc_file):
        if UPB_EXCLUDE_CC_FILES_PATTERN not in cc_file:
            return True
        if cc_file in UPB_EXCLUDE_CC_FILES_EXCEPTIONS:
            return True
        return False

    cc_files = [f for f in cc_files if _should_include(f)]

    # Collect list of .proto files that will be bundled in the grpcio_tools package.
    raw_proto_files = []
    for target in BAZEL_DEPS_COMMON_PROTOS_QUERIES:
        raw_proto_files += _bazel_query(target)
    proto_files = [
        name[len(PROTOBUF_PROTO_PREFIX) :].replace(":", "/")
        for name in raw_proto_files
        if name.endswith(".proto") and name.startswith(PROTOBUF_PROTO_PREFIX)
    ]

    commit_hash = protobuf_submodule_commit_hash()
    commit_hash_expr = COMMIT_HASH_PREFIX + commit_hash + COMMIT_HASH_SUFFIX

    deps_file_content = DEPS_FILE_CONTENT.format(
        cc_files=_pretty_print_list(sorted(cc_files)),
        proto_files=_pretty_print_list(sorted(set(proto_files))),
        cc_includes=_pretty_print_list(CC_INCLUDES),
        proto_include=repr(PROTO_INCLUDE),
        commit_hash_expr=commit_hash_expr,
    )
    return deps_file_content


def _copy_source_tree(source, target):
    """Copies source directory to a given target directory."""
    print("Copying contents of %s to %s" % (source, target))
    # TODO(jtattermusch): It is unclear why this legacy code needs to copy
    # the source directory to the target via the following boilerplate.
    # Should this code be simplified?
    for source_dir, _, files in os.walk(source):
        target_dir = os.path.abspath(
            os.path.join(target, os.path.relpath(source_dir, source))
        )
        try:
            os.makedirs(target_dir)
        except OSError as error:
            if error.errno != errno.EEXIST:
                raise
        for relative_file in files:
            source_file = os.path.abspath(
                os.path.join(source_dir, relative_file)
            )
            target_file = os.path.abspath(
                os.path.join(target_dir, relative_file)
            )
            shutil.copyfile(source_file, target_file)


def _delete_source_tree(target):
    """Deletes the copied target directory."""
    target = GRPCIO_TOOLS_ROOT_PREFIX + target
    target_abs = os.path.join(*target.split("/"))
    print("Deleting copied folder %s" % (target_abs))
    shutil.rmtree(target_abs, ignore_errors=True)


def main():
    parser = argparse.ArgumentParser()
    # In Step 1 below, the third_party folder is copied to a location required
    # by the build scripts. This folder does not need to be committed to the
    # repo, so you can pass `--cleanup_third_party` in automated scripts to
    # ensure that the temporary folders are deleted after the script runs.
    # See Jan's TODO in _copy_source_tree above.
    parser.add_argument(
        "--cleanup_third_party",
        default=False,
        action="store_true",
        help="Delete the temporary third_party folder",
    )
    args = parser.parse_args()
    os.chdir(GRPC_ROOT)

    # Step 1:
    # In order to be able to build the grpcio_tools package, we need the source code for the codegen plugins
    # and its dependencies to be available under the build root of the grpcio_tools package.
    # So we simply copy all the necessary files where the build will expect them to be.
    for source, target in COPY_FILES_SOURCE_TARGET_PAIRS:
        # convert the slashes in the relative path to platform-specific path dividers.
        # All paths are relative to GRPC_ROOT
        source_abs = os.path.join(GRPC_ROOT, os.path.join(*source.split("/")))
        # for targets, add grpcio_tools root prefix
        target = GRPCIO_TOOLS_ROOT_PREFIX + target
        target_abs = os.path.join(GRPC_ROOT, os.path.join(*target.split("/")))

        _copy_source_tree(source_abs, target_abs)
    print(
        "The necessary source files were copied under the grpcio_tools package"
        " root."
    )
    print()

    # Step 2:
    # Extract build metadata from bazel build (by running "bazel query")
    # and populate the protoc_lib_deps.py file with python-readable data structure
    # that will be used by grpcio_tools's setup.py (so it knows how to configure
    # the native build for the codegen plugin)
    try:
        print('Invoking "bazel query" to gather the protobuf dependencies.')
        protoc_lib_deps_content = _generate_deps_file_content()
    except Exception as error:
        # We allow this script to succeed even if we couldn't get the dependencies,
        # as then we can assume that even without a successful bazel run the
        # dependencies currently in source control are 'good enough'.
        sys.stderr.write("Got non-fatal error:\n")
        traceback.print_exc(file=sys.stderr)
        return
    # If we successfully got the dependencies, truncate and rewrite the deps file.
    with open(GRPC_PYTHON_PROTOC_LIB_DEPS, "w") as deps_file:
        deps_file.write(protoc_lib_deps_content)
    print('File "%s" updated.' % GRPC_PYTHON_PROTOC_LIB_DEPS)
    if args.cleanup_third_party:
        for target in DELETE_TARGETS_ON_CLEANUP:
            _delete_source_tree(target)
    print("Done.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/adaptive_concurrency/v3/adaptive_concurrency.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/adaptive_concurrency/v3/adaptive_concurrency.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.type.v3 import http_status_pb2 as envoy_dot_type_dot_v3_dot_http__status__pb2
from envoy.type.v3 import percent_pb2 as envoy_dot_type_dot_v3_dot_percent__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nPenvoy/extensions/filters/http/adaptive_concurrency/v3/adaptive_concurrency.proto\x12\x35\x65nvoy.extensions.filters.http.adaptive_concurrency.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x1f\x65nvoy/type/v3/http_status.proto\x1a\x1b\x65nvoy/type/v3/percent.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xea\t\n\x18GradientControllerConfig\x12;\n\x1bsample_aggregate_percentile\x18\x01 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12\x9d\x01\n\x18\x63oncurrency_limit_params\x18\x02 \x01(\x0b\x32q.envoy.extensions.filters.http.adaptive_concurrency.v3.GradientControllerConfig.ConcurrencyLimitCalculationParamsB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x92\x01\n\x13min_rtt_calc_params\x18\x03 \x01(\x0b\x32k.envoy.extensions.filters.http.adaptive_concurrency.v3.GradientControllerConfig.MinimumRTTCalculationParamsB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x1a\xae\x02\n!ConcurrencyLimitCalculationParams\x12\x44\n\x15max_concurrency_limit\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02 \x00\x12J\n\x1b\x63oncurrency_update_interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationB\n\xfa\x42\x07\xaa\x01\x04\x08\x01*\x00:w\x9a\xc5\x88\x1er\npenvoy.config.filter.http.adaptive_concurrency.v2alpha.GradientControllerConfig.ConcurrencyLimitCalculationParams\x1a\xd3\x03\n\x1bMinimumRTTCalculationParams\x12\x39\n\x08interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x0c\xfa\x42\t\xaa\x01\x06\x32\x04\x10\xc0\x84=\x12\x38\n\x0b\x66ixed_value\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02*\x00\x12<\n\rrequest_count\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02 \x00\x12&\n\x06jitter\x18\x03 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12>\n\x0fmin_concurrency\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02 \x00\x12&\n\x06\x62uffer\x18\x05 \x01(\x0b\x32\x16.envoy.type.v3.Percent:q\x9a\xc5\x88\x1el\njenvoy.config.filter.http.adaptive_concurrency.v2alpha.GradientControllerConfig.MinimumRTTCalculationParams:U\x9a\xc5\x88\x1eP\nNenvoy.config.filter.http.adaptive_concurrency.v2alpha.GradientControllerConfig\"\x8f\x03\n\x13\x41\x64\x61ptiveConcurrency\x12\x7f\n\x1agradient_controller_config\x18\x01 \x01(\x0b\x32O.envoy.extensions.filters.http.adaptive_concurrency.v3.GradientControllerConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00\x12\x39\n\x07\x65nabled\x18\x02 \x01(\x0b\x32(.envoy.config.core.v3.RuntimeFeatureFlag\x12\x44\n!concurrency_limit_exceeded_status\x18\x03 \x01(\x0b\x32\x19.envoy.type.v3.HttpStatus:P\x9a\xc5\x88\x1eK\nIenvoy.config.filter.http.adaptive_concurrency.v2alpha.AdaptiveConcurrencyB$\n\x1d\x63oncurrency_controller_config\x12\x03\xf8\x42\x01\x42\xde\x01\nCio.envoyproxy.envoy.extensions.filters.http.adaptive_concurrency.v3B\x18\x41\x64\x61ptiveConcurrencyProtoP\x01Zsgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/adaptive_concurrency/v3;adaptive_concurrencyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.adaptive_concurrency.v3.adaptive_concurrency_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nCio.envoyproxy.envoy.extensions.filters.http.adaptive_concurrency.v3B\030AdaptiveConcurrencyProtoP\001Zsgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/adaptive_concurrency/v3;adaptive_concurrencyv3\272\200\310\321\006\002\020\002'
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS'].fields_by_name['max_concurrency_limit']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS'].fields_by_name['max_concurrency_limit']._serialized_options = b'\372B\004*\002 \000'
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS'].fields_by_name['concurrency_update_interval']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS'].fields_by_name['concurrency_update_interval']._serialized_options = b'\372B\007\252\001\004\010\001*\000'
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS']._serialized_options = b'\232\305\210\036r\npenvoy.config.filter.http.adaptive_concurrency.v2alpha.GradientControllerConfig.ConcurrencyLimitCalculationParams'
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['interval']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['interval']._serialized_options = b'\372B\t\252\001\0062\004\020\300\204='
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['fixed_value']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['fixed_value']._serialized_options = b'\372B\005\252\001\002*\000'
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['request_count']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['request_count']._serialized_options = b'\372B\004*\002 \000'
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['min_concurrency']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS'].fields_by_name['min_concurrency']._serialized_options = b'\372B\004*\002 \000'
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS']._serialized_options = b'\232\305\210\036l\njenvoy.config.filter.http.adaptive_concurrency.v2alpha.GradientControllerConfig.MinimumRTTCalculationParams'
  _globals['_GRADIENTCONTROLLERCONFIG'].fields_by_name['concurrency_limit_params']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG'].fields_by_name['concurrency_limit_params']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_GRADIENTCONTROLLERCONFIG'].fields_by_name['min_rtt_calc_params']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG'].fields_by_name['min_rtt_calc_params']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_GRADIENTCONTROLLERCONFIG']._loaded_options = None
  _globals['_GRADIENTCONTROLLERCONFIG']._serialized_options = b'\232\305\210\036P\nNenvoy.config.filter.http.adaptive_concurrency.v2alpha.GradientControllerConfig'
  _globals['_ADAPTIVECONCURRENCY'].oneofs_by_name['concurrency_controller_config']._loaded_options = None
  _globals['_ADAPTIVECONCURRENCY'].oneofs_by_name['concurrency_controller_config']._serialized_options = b'\370B\001'
  _globals['_ADAPTIVECONCURRENCY'].fields_by_name['gradient_controller_config']._loaded_options = None
  _globals['_ADAPTIVECONCURRENCY'].fields_by_name['gradient_controller_config']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ADAPTIVECONCURRENCY']._loaded_options = None
  _globals['_ADAPTIVECONCURRENCY']._serialized_options = b'\232\305\210\036K\nIenvoy.config.filter.http.adaptive_concurrency.v2alpha.AdaptiveConcurrency'
  _globals['_GRADIENTCONTROLLERCONFIG']._serialized_start=390
  _globals['_GRADIENTCONTROLLERCONFIG']._serialized_end=1648
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS']._serialized_start=789
  _globals['_GRADIENTCONTROLLERCONFIG_CONCURRENCYLIMITCALCULATIONPARAMS']._serialized_end=1091
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS']._serialized_start=1094
  _globals['_GRADIENTCONTROLLERCONFIG_MINIMUMRTTCALCULATIONPARAMS']._serialized_end=1561
  _globals['_ADAPTIVECONCURRENCY']._serialized_start=1651
  _globals['_ADAPTIVECONCURRENCY']._serialized_end=2050
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/rbac/v3/rbac.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/rbac/v3/rbac.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.rbac.v3 import rbac_pb2 as envoy_dot_config_dot_rbac_dot_v3_dot_rbac__pb2
from xds.type.matcher.v3 import matcher_pb2 as xds_dot_type_dot_matcher_dot_v3_dot_matcher__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0envoy/extensions/filters/http/rbac/v3/rbac.proto\x12%envoy.extensions.filters.http.rbac.v3\x1a\x1f\x65nvoy/config/rbac/v3/rbac.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xc3\x03\n\x04RBAC\x12\x42\n\x05rules\x18\x01 \x01(\x0b\x32\x1a.envoy.config.rbac.v3.RBACB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0frules_specifier\x12\x19\n\x11rules_stat_prefix\x18\x06 \x01(\t\x12\x46\n\x07matcher\x18\x04 \x01(\x0b\x32\x1c.xds.type.matcher.v3.MatcherB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0frules_specifier\x12P\n\x0cshadow_rules\x18\x02 \x01(\x0b\x32\x1a.envoy.config.rbac.v3.RBACB\x1e\xf2\x98\xfe\x8f\x05\x18\x12\x16shadow_rules_specifier\x12T\n\x0eshadow_matcher\x18\x05 \x01(\x0b\x32\x1c.xds.type.matcher.v3.MatcherB\x1e\xf2\x98\xfe\x8f\x05\x18\x12\x16shadow_rules_specifier\x12 \n\x18shadow_rules_stat_prefix\x18\x03 \x01(\t\x12\x1c\n\x14track_per_rule_stats\x18\x07 \x01(\x08:,\x9a\xc5\x88\x1e\'\n%envoy.config.filter.http.rbac.v2.RBAC\"\x85\x01\n\x0cRBACPerRoute\x12\x39\n\x04rbac\x18\x02 \x01(\x0b\x32+.envoy.extensions.filters.http.rbac.v3.RBAC:4\x9a\xc5\x88\x1e/\n-envoy.config.filter.http.rbac.v2.RBACPerRouteJ\x04\x08\x01\x10\x02\x42\x9f\x01\n3io.envoyproxy.envoy.extensions.filters.http.rbac.v3B\tRbacProtoP\x01ZSgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/rbac/v3;rbacv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.rbac.v3.rbac_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n3io.envoyproxy.envoy.extensions.filters.http.rbac.v3B\tRbacProtoP\001ZSgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/rbac/v3;rbacv3\272\200\310\321\006\002\020\002'
  _globals['_RBAC'].fields_by_name['rules']._loaded_options = None
  _globals['_RBAC'].fields_by_name['rules']._serialized_options = b'\362\230\376\217\005\021\022\017rules_specifier'
  _globals['_RBAC'].fields_by_name['matcher']._loaded_options = None
  _globals['_RBAC'].fields_by_name['matcher']._serialized_options = b'\362\230\376\217\005\021\022\017rules_specifier'
  _globals['_RBAC'].fields_by_name['shadow_rules']._loaded_options = None
  _globals['_RBAC'].fields_by_name['shadow_rules']._serialized_options = b'\362\230\376\217\005\030\022\026shadow_rules_specifier'
  _globals['_RBAC'].fields_by_name['shadow_matcher']._loaded_options = None
  _globals['_RBAC'].fields_by_name['shadow_matcher']._serialized_options = b'\362\230\376\217\005\030\022\026shadow_rules_specifier'
  _globals['_RBAC']._loaded_options = None
  _globals['_RBAC']._serialized_options = b'\232\305\210\036\'\n%envoy.config.filter.http.rbac.v2.RBAC'
  _globals['_RBACPERROUTE']._loaded_options = None
  _globals['_RBACPERROUTE']._serialized_options = b'\232\305\210\036/\n-envoy.config.filter.http.rbac.v2.RBACPerRoute'
  _globals['_RBAC']._serialized_start=258
  _globals['_RBAC']._serialized_end=709
  _globals['_RBACPERROUTE']._serialized_start=712
  _globals['_RBACPERROUTE']._serialized_end=845
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import threading

from grpc._cython import cygrpc


class SimpleFuture:
    """A simple future mechanism."""

    def __init__(self, function, *args, **kwargs):
        def wrapped_function():
            try:
                self._result = function(*args, **kwargs)
            except Exception as error:  # pylint: disable=broad-except
                self._error = error

        self._result = None
        self._error = None
        self._thread = threading.Thread(target=wrapped_function)
        self._thread.start()

    def result(self):
        """The resulting value of this future.

        Re-raises any exceptions.
        """
        self._thread.join()
        if self._error:
            # TODO(atash): re-raise exceptions in a way that preserves tracebacks
            raise self._error  # pylint: disable=raising-bad-type
        return self._result


class CompletionQueuePollFuture(SimpleFuture):
    def __init__(self, completion_queue, deadline):
        super(CompletionQueuePollFuture, self).__init__(
            lambda: completion_queue.poll(deadline=deadline)
        )



#!/usr/bin/env python3
#
# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import csv
import glob
import math
import multiprocessing
import os
import pathlib
import re
import shutil
import subprocess
import sys

sys.path.append(
    os.path.join(
        os.path.dirname(sys.argv[0]), "..", "..", "run_tests", "python_utils"
    )
)
import check_on_pr

argp = argparse.ArgumentParser(description="Perform diff on memory benchmarks")

argp.add_argument(
    "-d",
    "--diff_base",
    type=str,
    help="Commit or branch to compare the current one to",
)

argp.add_argument("-j", "--jobs", type=int, default=multiprocessing.cpu_count())

args = argp.parse_args()

_INTERESTING = {
    "call/client": (
        rb"client call memory usage: ([0-9\.]+) bytes per call",
        float,
    ),
    "call/server": (
        rb"server call memory usage: ([0-9\.]+) bytes per call",
        float,
    ),
    "channel/client": (
        rb"client channel memory usage: ([0-9\.]+) bytes per channel",
        float,
    ),
    "channel/server": (
        rb"server channel memory usage: ([0-9\.]+) bytes per channel",
        float,
    ),
    "call/xds_client": (
        rb"xds client call memory usage: ([0-9\.]+) bytes per call",
        float,
    ),
    "call/xds_server": (
        rb"xds server call memory usage: ([0-9\.]+) bytes per call",
        float,
    ),
    "channel/xds_client": (
        rb"xds client channel memory usage: ([0-9\.]+) bytes per channel",
        float,
    ),
    "channel/xds_server": (
        rb"xds server channel memory usage: ([0-9\.]+) bytes per channel",
        float,
    ),
    "channel_multi_address/xds_client": (
        rb"xds multi_address client channel memory usage: ([0-9\.]+) bytes per channel",
        float,
    ),
}

_SCENARIOS = {
    "default": [],
    "minstack": ["--scenario_config=minstack"],
    # TODO(aananthv)(b/500562759): Figure out why chaotic good test cases are
    # taking time.
    # The reason for disabling:
    # For the call Benchmark (server.cc and client.cc)
    # Client: Uses Chaotic Good transport. It processes the --chaotic_good flag
    # and adds the GRPC_ARG_PREFERRED_TRANSPORT_PROTOCOLS argument to args_vec
    # before creating the channel.
    #
    # Server: Falls back to chttp2 (due to a bug). The server binary receives
    # the --chaotic_good flag from the runner, but it pushes the
    # GRPC_ARG_PREFERRED_TRANSPORT_PROTOCOLS argument to args_vec after it has
    # already called grpc_server_create with the older snapshot of arguments.
    #
    # "chaotic_good": ["--scenario_config=chaotic_good"],
}

_BENCHMARKS = {
    "call": ["--benchmark_names=call", "--size=50000"],
    "channel": ["--benchmark_names=channel", "--size=10000"],
    "channel_multi_address": [
        "--benchmark_names=channel_multi_address",
        "--size=10000",
    ],
}


def _run():
    """Build with Bazel, then run, and extract interesting lines from the output."""
    subprocess.check_call(
        [
            "tools/bazel",
            "build",
            "-c",
            "opt",
            "test/core/memory_usage/memory_usage_test",
        ]
    )
    ret = {}
    for name, benchmark_args in _BENCHMARKS.items():
        for use_xds in (False, True):
            for scenario, extra_args in _SCENARIOS.items():
                # TODO(chenancy) Remove when minstack is implemented for channel
                if name == "channel" and scenario == "minstack":
                    continue
                if name == "channel_multi_address" and not use_xds:
                    continue
                argv = (
                    ["bazel-bin/test/core/memory_usage/memory_usage_test"]
                    + benchmark_args
                    + extra_args
                )
                if use_xds:
                    argv.append("--use_xds")
                try:
                    output = subprocess.check_output(argv)
                except subprocess.CalledProcessError as e:
                    print("Error running benchmark:", e)
                    continue
                for line in output.splitlines():
                    for key, (pattern, conversion) in _INTERESTING.items():
                        m = re.match(pattern, line)
                        if m:
                            ret[scenario + ": " + key] = conversion(m.group(1))
    return ret


cur = _run()
old = None

if args.diff_base:
    where_am_i = (
        subprocess.check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"])
        .decode()
        .strip()
    )
    # checkout the diff base (="old")
    subprocess.check_call(["git", "checkout", args.diff_base])
    try:
        old = _run()
    finally:
        # restore the original revision (="cur")
        subprocess.check_call(["git", "checkout", where_am_i])

text = ""
if old is None:
    print(cur)
    for key, value in sorted(cur.items()):
        text += "{}: {}\n".format(key, value)
else:
    print(cur, old)
    call_decrease = 0
    call_increase = 0
    channel_decrease = 0
    channel_increase = 0
    call_significant = 200
    channel_significant = 1000
    for scenario in _SCENARIOS.keys():
        for key, value in sorted(_INTERESTING.items()):
            key = scenario + ": " + key
            if key in cur:
                if key not in old:
                    text += "{}: {}\n".format(key, cur[key])
                else:
                    text += "{}: {} -> {}\n".format(key, old[key], cur[key])
                    diff = cur[key] - old[key]
                    if "call" in key:
                        if diff < -call_significant:
                            call_decrease += 1
                        elif diff > call_significant:
                            call_increase += 1
                    else:
                        if diff < -channel_significant:
                            channel_decrease += 1
                        elif diff > channel_significant:
                            channel_increase += 1

    print("CALL: %d increases, %d decreases" % (call_increase, call_decrease))
    print(
        "CHANNEL: %d increases, %d decreases"
        % (channel_increase, channel_decrease)
    )
    # if anything increased ==> label it an increase
    # otherwise if anything decreased (and nothing increased) ==> label it a decrease
    # otherwise label it neutral
    # -- this biases reporting towards increases, which is what we want to act on
    #    most regularly
    if call_increase > 0:
        check_on_pr.label_increase_decrease_on_pr("per-call-memory", 1)
    elif call_decrease > 0:
        check_on_pr.label_increase_decrease_on_pr("per-call-memory", -1)
    else:
        check_on_pr.label_increase_decrease_on_pr("per-call-memory", 0)
    if channel_increase > 0:
        check_on_pr.label_increase_decrease_on_pr("per-channel-memory", 1)
    elif channel_decrease > 0:
        check_on_pr.label_increase_decrease_on_pr("per-channel-memory", -1)
    else:
        check_on_pr.label_increase_decrease_on_pr("per-channel-memory", 0)

print(text)
check_on_pr.check_on_pr("Memory Difference", "```\n%s\n```" % text)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Abstract base classes for Channel objects and Multicallable objects."""

import abc
from typing import Generic, Optional

import grpc

from . import _base_call
from ._typing import DeserializingFunction
from ._typing import MetadataType
from ._typing import RequestIterableType
from ._typing import RequestType
from ._typing import ResponseType
from ._typing import SerializingFunction


class UnaryUnaryMultiCallable(Generic[RequestType, ResponseType], abc.ABC):
    """Enables asynchronous invocation of a unary-call RPC."""

    @abc.abstractmethod
    def __call__(
        self,
        request: RequestType,
        *,
        timeout: Optional[float] = None,
        metadata: Optional[MetadataType] = None,
        credentials: Optional[grpc.CallCredentials] = None,
        wait_for_ready: Optional[bool] = None,
        compression: Optional[grpc.Compression] = None,
    ) -> _base_call.UnaryUnaryCall[RequestType, ResponseType]:
        """Asynchronously invokes the underlying RPC.

        Args:
          request: The request value for the RPC.
          timeout: An optional duration of time in seconds to allow
            for the RPC.
          metadata: Optional :term:`metadata` to be transmitted to the
            service-side of the RPC.
          credentials: An optional CallCredentials for the RPC. Only valid for
            secure Channel.
          wait_for_ready: An optional flag to enable :term:`wait_for_ready` mechanism.
          compression: An element of grpc.Compression, e.g.
            grpc.Compression.Gzip.

        Returns:
          A UnaryUnaryCall object.

        Raises:
          RpcError: Indicates that the RPC terminated with non-OK status. The
            raised RpcError will also be a Call for the RPC affording the RPC's
            metadata, status code, and details.
        """


class UnaryStreamMultiCallable(Generic[RequestType, ResponseType], abc.ABC):
    """Enables asynchronous invocation of a server-streaming RPC."""

    @abc.abstractmethod
    def __call__(
        self,
        request: RequestType,
        *,
        timeout: Optional[float] = None,
        metadata: Optional[MetadataType] = None,
        credentials: Optional[grpc.CallCredentials] = None,
        wait_for_ready: Optional[bool] = None,
        compression: Optional[grpc.Compression] = None,
    ) -> _base_call.UnaryStreamCall[RequestType, ResponseType]:
        """Asynchronously invokes the underlying RPC.

        Args:
          request: The request value for the RPC.
          timeout: An optional duration of time in seconds to allow
            for the RPC.
          metadata: Optional :term:`metadata` to be transmitted to the
            service-side of the RPC.
          credentials: An optional CallCredentials for the RPC. Only valid for
            secure Channel.
          wait_for_ready: An optional flag to enable :term:`wait_for_ready` mechanism.
          compression: An element of grpc.Compression, e.g.
            grpc.Compression.Gzip.

        Returns:
          A UnaryStreamCall object.

        Raises:
          RpcError: Indicates that the RPC terminated with non-OK status. The
            raised RpcError will also be a Call for the RPC affording the RPC's
            metadata, status code, and details.
        """


class StreamUnaryMultiCallable(abc.ABC):
    """Enables asynchronous invocation of a client-streaming RPC."""

    @abc.abstractmethod
    def __call__(
        self,
        request_iterator: Optional[RequestIterableType] = None,
        timeout: Optional[float] = None,
        metadata: Optional[MetadataType] = None,
        credentials: Optional[grpc.CallCredentials] = None,
        wait_for_ready: Optional[bool] = None,
        compression: Optional[grpc.Compression] = None,
    ) -> _base_call.StreamUnaryCall:
        """Asynchronously invokes the underlying RPC.

        Args:
          request_iterator: An optional async iterable or iterable of request
            messages for the RPC.
          timeout: An optional duration of time in seconds to allow
            for the RPC.
          metadata: Optional :term:`metadata` to be transmitted to the
            service-side of the RPC.
          credentials: An optional CallCredentials for the RPC. Only valid for
            secure Channel.
          wait_for_ready: An optional flag to enable :term:`wait_for_ready` mechanism.
          compression: An element of grpc.Compression, e.g.
            grpc.Compression.Gzip.

        Returns:
          A StreamUnaryCall object.

        Raises:
          RpcError: Indicates that the RPC terminated with non-OK status. The
            raised RpcError will also be a Call for the RPC affording the RPC's
            metadata, status code, and details.
        """


class StreamStreamMultiCallable(abc.ABC):
    """Enables asynchronous invocation of a bidirectional-streaming RPC."""

    @abc.abstractmethod
    def __call__(
        self,
        request_iterator: Optional[RequestIterableType] = None,
        timeout: Optional[float] = None,
        metadata: Optional[MetadataType] = None,
        credentials: Optional[grpc.CallCredentials] = None,
        wait_for_ready: Optional[bool] = None,
        compression: Optional[grpc.Compression] = None,
    ) -> _base_call.StreamStreamCall:
        """Asynchronously invokes the underlying RPC.

        Args:
          request_iterator: An optional async iterable or iterable of request
            messages for the RPC.
          timeout: An optional duration of time in seconds to allow
            for the RPC.
          metadata: Optional :term:`metadata` to be transmitted to the
            service-side of the RPC.
          credentials: An optional CallCredentials for the RPC. Only valid for
            secure Channel.
          wait_for_ready: An optional flag to enable :term:`wait_for_ready` mechanism.
          compression: An element of grpc.Compression, e.g.
            grpc.Compression.Gzip.

        Returns:
          A StreamStreamCall object.

        Raises:
          RpcError: Indicates that the RPC terminated with non-OK status. The
            raised RpcError will also be a Call for the RPC affording the RPC's
            metadata, status code, and details.
        """


class Channel(abc.ABC):
    """Enables asynchronous RPC invocation as a client.

    Channel objects implement the Asynchronous Context Manager (aka. async
    with) type, although they are not supported to be entered and exited
    multiple times.
    """

    @abc.abstractmethod
    async def __aenter__(self):
        """Starts an asynchronous context manager.

        Returns:
          Channel the channel that was instantiated.
        """

    @abc.abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Finishes the asynchronous context manager by closing the channel.

        Still active RPCs will be cancelled.
        """

    @abc.abstractmethod
    async def close(self, grace: Optional[float] = None):
        """Closes this Channel and releases all resources held by it.

        This method immediately stops the channel from executing new RPCs in
        all cases.

        If a grace period is specified, this method waits until all active
        RPCs are finished or until the grace period is reached. RPCs that haven't
        been terminated within the grace period are aborted.
        If a grace period is not specified (by passing None for grace),
        all existing RPCs are cancelled immediately.

        This method is idempotent.
        """

    @abc.abstractmethod
    def get_state(
        self, try_to_connect: bool = False
    ) -> grpc.ChannelConnectivity:
        """Checks the connectivity state of a channel.

        This is an EXPERIMENTAL API.

        If the channel reaches a stable connectivity state, it is guaranteed
        that the return value of this function will eventually converge to that
        state.

        Args:
          try_to_connect: a bool indicate whether the Channel should try to
            connect to peer or not.

        Returns: A ChannelConnectivity object.
        """

    @abc.abstractmethod
    async def wait_for_state_change(
        self,
        last_observed_state: grpc.ChannelConnectivity,
    ) -> None:
        """Waits for a change in connectivity state.

        This is an EXPERIMENTAL API.

        The function blocks until there is a change in the channel connectivity
        state from the "last_observed_state". If the state is already
        different, this function will return immediately.

        There is an inherent race between the invocation of
        "Channel.wait_for_state_change" and "Channel.get_state". The state can
        change arbitrary many times during the race, so there is no way to
        observe every state transition.

        If there is a need to put a timeout for this function, please refer to
        "asyncio.wait_for".

        Args:
          last_observed_state: A grpc.ChannelConnectivity object representing
            the last known state.
        """

    @abc.abstractmethod
    async def channel_ready(self) -> None:
        """Creates a coroutine that blocks until the Channel is READY."""

    @abc.abstractmethod
    def unary_unary(
        self,
        method: str,
        request_serializer: Optional[SerializingFunction] = None,
        response_deserializer: Optional[DeserializingFunction] = None,
        _registered_method: Optional[bool] = False,
    ) -> UnaryUnaryMultiCallable:
        """Creates a UnaryUnaryMultiCallable for a unary-unary method.

        Args:
          method: The name of the RPC method.
          request_serializer: Optional :term:`serializer` for serializing the request
            message. Request goes unserialized in case None is passed.
          response_deserializer: Optional :term:`deserializer` for deserializing the
            response message. Response goes undeserialized in case None
            is passed.
          _registered_method: Implementation Private. Optional: A bool representing
            whether the method is registered.

        Returns:
          A UnaryUnaryMultiCallable value for the named unary-unary method.
        """

    @abc.abstractmethod
    def unary_stream(
        self,
        method: str,
        request_serializer: Optional[SerializingFunction] = None,
        response_deserializer: Optional[DeserializingFunction] = None,
        _registered_method: Optional[bool] = False,
    ) -> UnaryStreamMultiCallable:
        """Creates a UnaryStreamMultiCallable for a unary-stream method.

        Args:
          method: The name of the RPC method.
          request_serializer: Optional :term:`serializer` for serializing the request
            message. Request goes unserialized in case None is passed.
          response_deserializer: Optional :term:`deserializer` for deserializing the
            response message. Response goes undeserialized in case None
            is passed.
          _registered_method: Implementation Private. Optional: A bool representing
            whether the method is registered.

        Returns:
          A UnaryStreamMultiCallable value for the named unary-stream method.
        """

    @abc.abstractmethod
    def stream_unary(
        self,
        method: str,
        request_serializer: Optional[SerializingFunction] = None,
        response_deserializer: Optional[DeserializingFunction] = None,
        _registered_method: Optional[bool] = False,
    ) -> StreamUnaryMultiCallable:
        """Creates a StreamUnaryMultiCallable for a stream-unary method.

        Args:
          method: The name of the RPC method.
          request_serializer: Optional :term:`serializer` for serializing the request
            message. Request goes unserialized in case None is passed.
          response_deserializer: Optional :term:`deserializer` for deserializing the
            response message. Response goes undeserialized in case None
            is passed.
          _registered_method: Implementation Private. Optional: A bool representing
            whether the method is registered.

        Returns:
          A StreamUnaryMultiCallable value for the named stream-unary method.
        """

    @abc.abstractmethod
    def stream_stream(
        self,
        method: str,
        request_serializer: Optional[SerializingFunction] = None,
        response_deserializer: Optional[DeserializingFunction] = None,
        _registered_method: Optional[bool] = False,
    ) -> StreamStreamMultiCallable:
        """Creates a StreamStreamMultiCallable for a stream-stream method.

        Args:
          method: The name of the RPC method.
          request_serializer: Optional :term:`serializer` for serializing the request
            message. Request goes unserialized in case None is passed.
          response_deserializer: Optional :term:`deserializer` for deserializing the
            response message. Response goes undeserialized in case None
            is passed.
          _registered_method: Implementation Private. Optional: A bool representing
            whether the method is registered.

        Returns:
          A StreamStreamMultiCallable value for the named stream-stream method.
        """
